@@ -260,7 +260,7 @@ static int cos_cache_add_tmpl(cosTemplates **pTemplates, cosAttrValue *dn, cosAt
 
 /* cosDefinitions manipulation */
 static int cos_cache_build_definition_list(cosDefinitions **pDefs, int *vattr_cacheable);
-static int cos_cache_add_dn_defs(char *dn, cosDefinitions **pDefs, int *vattr_cacheable);
+static int cos_cache_add_dn_defs(char *dn, cosDefinitions **pDefs);
 static int cos_cache_add_defn(cosDefinitions **pDefs, cosAttrValue **dn, int cosType, cosAttrValue **tree, cosAttrValue **tmpDn, cosAttrValue **spec, cosAttrValue **pAttrs, cosAttrValue **pOverrides, cosAttrValue **pOperational, cosAttrValue **pCosMerge, cosAttrValue **pCosOpDefault);
 static int cos_cache_entry_is_cos_related( Slapi_Entry *e);
 
@@ -619,9 +619,9 @@ static int cos_cache_build_definition_list(cosDefinitions **pDefs, int *vattr_ca
 	LDAPDebug( LDAP_DEBUG_TRACE, "--> cos_cache_build_definition_list\n",0,0,0);
 
 	/*
-		the class of service definitions may be anywhere in the DIT,
-		so our first task is to find them.
-	*/
+	 * The class of service definitions may be anywhere in the DIT,
+	 * so our first task is to find them.
+	 */
 
 	attrs[0] = "namingcontexts";
 	attrs[1] = 0;
@@ -629,9 +629,9 @@ static int cos_cache_build_definition_list(cosDefinitions **pDefs, int *vattr_ca
 	LDAPDebug( LDAP_DEBUG_PLUGIN, "cos: Building class of service cache after status change.\n",0,0,0);
 
 	/*
-	 * XXXrbyrne: this looks really ineficient--should be using
+	 * XXXrbyrne: this looks really inefficient--should be using
 	 * slapi_get_next_suffix(), rather than searching for namingcontexts.
-	*/
+	 */
 
 	pSuffixSearch = slapi_search_internal("",LDAP_SCOPE_BASE,"(objectclass=*)",NULL,attrs,0);
 	if(pSuffixSearch)
@@ -671,19 +671,21 @@ static int cos_cache_build_definition_list(cosDefinitions **pDefs, int *vattr_ca
 							{
 								/* here's a suffix, lets search it... */
 								if(suffixVals[valIndex]->bv_val)
-									if(!cos_cache_add_dn_defs(suffixVals[valIndex]->bv_val ,pDefs, vattr_cacheable))
+								{
+									if(!cos_cache_add_dn_defs(suffixVals[valIndex]->bv_val ,pDefs))
+									{
+										*vattr_cacheable = -1;
 										cos_def_available = 1;
-								
+										break;
+									}
+								}
 								valIndex++;
 							}
-
-
 							ber_bvecfree( suffixVals );
 							suffixVals = NULL;
 						}
 					}
 				}
-
 			} while(!slapi_entry_next_attr(pSuffixList[suffixIndex], suffixAttr, &suffixAttr));
 		}
 		suffixIndex++;
@@ -709,7 +711,6 @@ next:
 		slapi_pblock_destroy(pSuffixSearch);
 	}
 
-
 	LDAPDebug( LDAP_DEBUG_TRACE, "<-- cos_cache_build_definition_list\n",0,0,0);
 	return ret;
 }
@@ -725,9 +726,10 @@ struct dn_defs_info {
 /*
  * Currently, always returns 0 to continue the search for definitions, even
  * if a particular attempt to add a definition fails: info.ret gets set to
- * zero only if we succed to add a def.
+ * zero only if we succeed to add a def.
 */
-static int 	cos_dn_defs_cb (Slapi_Entry* e, void *callback_data)
+static int 
+cos_dn_defs_cb (Slapi_Entry* e, void *callback_data)
 {
 	struct dn_defs_info *info;
 	cosAttrValue **pSneakyVal = 0;
@@ -749,10 +751,6 @@ static int 	cos_dn_defs_cb (Slapi_Entry* e, void *callback_data)
 	char *norm_dn = NULL;
 	info=(struct dn_defs_info *)callback_data;
 	
-			
-	/* assume cacheable */
-	info->vattr_cacheable = -1;
-
 	cos_cache_add_attrval(&pDn, slapi_entry_get_dn(e));
 	if(slapi_entry_first_attr(e, &dnAttr)) {
 		goto bail;
@@ -877,31 +875,10 @@ static int 	cos_dn_defs_cb (Slapi_Entry* e, void *callback_data)
 					                      dnVals[valIndex]->bv_val);
 				}
 
-				if(!pCosTargetTree)
-				{
-					/* get the parent of the definition */
-					char *orig = slapi_dn_parent(pDn->val);
-					Slapi_DN *psdn = slapi_sdn_new_dn_byval(orig);
-					char *parent = (char *)slapi_sdn_get_dn(psdn);
-					if (!parent) {
-						parent = (char *)slapi_sdn_get_udn(psdn);
-						LDAPDebug(LDAP_DEBUG_ANY, 
-						  "cos_cache_build_definition_list: "
-						  "failed to normalize parent dn %s. "
-						  "Adding the pre normalized dn.\n", 
-						  parent, 0, 0);
-					}
-					cos_cache_add_attrval(&pCosTargetTree, parent);
-					if (!pCosTemplateDn) {
-						cos_cache_add_attrval(&pCosTemplateDn, parent);
-					}
-					slapi_sdn_free(&psdn);
-				}
-				
 				slapi_vattrspi_regattr((vattr_sp_handle *)vattr_handle,
 				                        dnVals[valIndex]->bv_val, NULL, NULL);
 			} /* if(attrType is cosAttribute) */
-										
+
 			/*
 			 * Add the attributetype to the appropriate
 			 * list.
@@ -913,6 +890,47 @@ static int 	cos_dn_defs_cb (Slapi_Entry* e, void *callback_data)
 		ber_bvecfree( dnVals );
 		dnVals = NULL;
 	} while(!slapi_entry_next_attr(e, dnAttr, &dnAttr));
+
+	if (pCosAttribute && (!pCosTargetTree || !pCosTemplateDn)) {
+		/* get the parent of the definition */
+		char *orig = slapi_dn_parent(pDn->val);
+		char *parent = NULL;
+		if (orig) {
+			parent = slapi_create_dn_string("%s", orig);
+			if (!parent) {
+				parent = orig;
+				LDAPDebug1Arg(LDAP_DEBUG_ANY, 
+				              "cos_dn_defs_cb: "
+				              "failed to normalize parent dn %s. "
+				              "Adding the pre normalized dn.\n", 
+				              parent);
+			}
+			if (!pCosTargetTree) {
+				cos_cache_add_attrval(&pCosTargetTree, parent);
+			}
+			if (!pCosTemplateDn) {
+				cos_cache_add_attrval(&pCosTemplateDn, parent);
+			}
+			if (parent != orig) {
+				slapi_ch_free_string(&parent);
+			}
+			slapi_ch_free_string(&orig);
+		} else {
+			LDAPDebug1Arg(LDAP_DEBUG_ANY, 
+			              "cos_dn_defs_cb: "
+			              "failed to get parent dn of cos definition %s.\n",
+			              pDn->val);
+			if (!pCosTemplateDn) {
+				if (!pCosTargetTree) {
+					LDAPDebug0Args(LDAP_DEBUG_ANY, "cosTargetTree and cosTemplateDn are not set.\n");
+				} else {
+					LDAPDebug0Args(LDAP_DEBUG_ANY, "cosTemplateDn is not set.\n");
+				}
+			} else if (!pCosTargetTree) {
+				LDAPDebug0Args(LDAP_DEBUG_ANY, "cosTargetTree is not set.\n");
+			}
+		}
+	}
 	
 	/*
 	determine the type of class of service scheme 
@@ -951,9 +969,7 @@ static int 	cos_dn_defs_cb (Slapi_Entry* e, void *callback_data)
 	*/
 	
 	/* these must exist */
-	if(		pDn &&
-		pObjectclass && 
-		
+	if(pDn && pObjectclass && 
 		(
 		(cosType == COSTYPE_CLASSIC &&
 		pCosTemplateDn && 
@@ -1037,10 +1053,10 @@ bail:
 	/* This particular definition may not have yielded anything
      * worth caching (eg. no template was found for it) but
      * that should not cause us to abort the search for other more well behaved
-     * definitions.
+     * definitions unless we are shutting down.
      * return info->ret;
-    */
-    return (0);
+     */
+    return slapi_is_shutting_down();
 
 }
 
@@ -1057,7 +1073,7 @@ bail:
 
 #define DN_DEF_FILTER "(&(|(objectclass=cosSuperDefinition)(objectclass=cosDefinition))(objectclass=ldapsubentry))"
 
-static int cos_cache_add_dn_defs(char *dn, cosDefinitions **pDefs, int *vattr_cacheable)
+static int cos_cache_add_dn_defs(char *dn, cosDefinitions **pDefs)
 {
 	Slapi_PBlock *pDnSearch = 0;
 	struct dn_defs_info info = {NULL, 0, 0};
@@ -1065,7 +1081,6 @@ static int cos_cache_add_dn_defs(char *dn, cosDefinitions **pDefs, int *vattr_ca
 	if (pDnSearch) {
 		info.ret=-1; /* assume no good defs */
 		info.pDefs=pDefs;
-		info.vattr_cacheable = 0; /* assume not cacheable */
 		slapi_search_internal_set_pb(pDnSearch, dn, LDAP_SCOPE_SUBTREE,
 									 DN_DEF_FILTER,NULL,0,
 									 NULL,NULL,cos_get_plugin_identity(),0);
@@ -1077,8 +1092,6 @@ static int cos_cache_add_dn_defs(char *dn, cosDefinitions **pDefs, int *vattr_ca
 		slapi_pblock_destroy (pDnSearch);
 	}
 
-	*vattr_cacheable = info.vattr_cacheable;
-
 	return info.ret;
 }
 
@@ -1097,7 +1110,7 @@ struct tmpl_info {
 /*
  * Currently, always returns 0 to continue the search for templates, even
  * if a particular attempt to add a template fails: info.ret gets set to
- * zero only if we succed to add at least one tmpl.
+ * zero only if we succeed to add at least one tmpl.
 */
 static int 	cos_dn_tmpl_entries_cb (Slapi_Entry* e, void *callback_data) {
 	cosAttrValue *pDn = 0;
@@ -1147,7 +1160,7 @@ static int 	cos_dn_tmpl_entries_cb (Slapi_Entry* e, void *callback_data) {
 				
 				if(pSneakyVal == NULL)
 				{
-					/* look for the atrribute in the dynamic attributes */
+					/* look for the attribute in the dynamic attributes */
 					if(cos_cache_attrval_exists(info->pAttrs, attrType))
 					{
 						pSneakyVal = &pCosAttribute;
@@ -1250,10 +1263,10 @@ static int 	cos_dn_tmpl_entries_cb (Slapi_Entry* e, void *callback_data) {
 		}
 	}			
 	/*
-	 * Always contine the search even if a particular attempt
-	 * to add a template failed.
-	*/	
-	return 0;
+	 * Always continue the search even if a particular attempt
+	 * to add a template failed unless we are shutting down
+	 */
+	return slapi_is_shutting_down();
 }
 
 /*
@@ -1844,6 +1857,8 @@ static void cos_cache_del_schema(cosCache *pCache)
 				cos_cache_del_attrval_list(&(pCache->ppAttrIndex[attr_index]->pObjectclasses));
 			}
 		}
+		/* Finally, remove the first attribute's objectclass list */
+		cos_cache_del_attrval_list(&(pCache->ppAttrIndex[0]->pObjectclasses));
 	}
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "<-- cos_cache_del_schema\n",0,0,0);
@@ -2096,7 +2111,7 @@ static int cos_cache_vattr_get(vattr_sp_handle *handle, vattr_context *c, Slapi_
 	ret = cos_cache_query_attr(pCache, c, e, type, results, NULL, NULL, NULL);
 	if(ret == 0)
 	{
-        *free_flags = SLAPI_VIRTUALATTRS_RETURNED_COPIES;
+        *free_flags = SLAPI_VIRTUALATTRS_RETURNED_COPIES | SLAPI_VIRTUALATTRS_VALUES_CACHEABLE;
         *actual_type_name = slapi_ch_strdup(type);
 		*type_name_disposition = SLAPI_VIRTUALATTRS_TYPE_NAME_MATCHED_EXACTLY_OR_ALIAS;
 	}
@@ -3033,25 +3048,19 @@ static int cos_cache_attr_compare(const void *e1, const void *e2)
 	int com_Result;
 	cosAttributes *pAttr = (*(cosAttributes**)e1);
 	cosTemplates *pTemplate = (cosTemplates*)pAttr->pParent;
-    cosDefinitions *pDef = (cosDefinitions*)pTemplate->pParent;
-	cosAttrValue *pcostt = pDef->pCosTargetTree;
 	cosAttributes *pAttr1 = (*(cosAttributes**)e2);
 	cosTemplates *pTemplate1 = (cosTemplates*)pAttr1->pParent;
-	cosDefinitions *pDef1 = (cosDefinitions*)pTemplate1->pParent;
-    cosAttrValue *pcostt1 = pDef1->pCosTargetTree;
         
 	/* Now compare the names of the attributes */
 	com_Result = slapi_utf8casecmp((unsigned char*)(*(cosAttributes**)e1)->pAttrName,(unsigned char*)(*(cosAttributes**)e2)->pAttrName);
+	if(0 == com_Result){
+		/* Now compare the cosPriorities */
+		com_Result = pTemplate->cosPriority - pTemplate1->cosPriority;
+	}
 	if(0 == com_Result)
-	/* Now compare the definition Dn parents */
-      com_Result = slapi_utf8casecmp((unsigned char*)pcostt1->val,(unsigned char*)pcostt->val);   
-	  if(0 == com_Result)
-	/* Now compare the cosPririoties */     
-	     com_Result = pTemplate->cosPriority - pTemplate1->cosPriority;
-	/* Now compare the prirority */ 
-		  if(0 == com_Result)
-	            return -1;   
-		  return com_Result;
+		return -1;
+
+	return com_Result;
 }
 
 static int cos_cache_string_compare(const void *e1, const void *e2)
@@ -3395,15 +3404,15 @@ void cos_cache_stop()
 
 	/* release the caches reference to the cache */
 	cos_cache_release(pCache);
-
-	slapi_destroy_mutex(cache_lock);
-	slapi_destroy_mutex(change_lock);
-	slapi_destroy_condvar(something_changed);
+	slapi_destroy_mutex(cache_lock); cache_lock = NULL;
+	slapi_destroy_mutex(change_lock); change_lock = NULL;
+	slapi_destroy_condvar(something_changed); something_changed = NULL;
 
 	slapi_unlock_mutex(stop_lock);
-	slapi_destroy_mutex(stop_lock);
-	slapi_destroy_condvar(start_cond);
-	slapi_destroy_mutex(start_lock);
+
+	slapi_destroy_mutex(stop_lock); stop_lock = NULL;
+	slapi_destroy_condvar(start_cond); start_cond = NULL;
+	slapi_destroy_mutex(start_lock); start_lock = NULL;
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "<-- cos_cache_stop\n",0,0,0);
 }
@@ -3582,14 +3591,9 @@ static int cos_cache_entry_is_cos_related( Slapi_Entry *e) {
 			{
 				pObj = (char*)slapi_value_get_string(val);
 
-				/*
-				 * objectclasses are ascii--maybe strcasecmp() is faster than
-				 * slapi_utf8casecmp()
-				*/
-				if(	!strcasecmp(pObj, "cosdefinition") ||
-					!strcasecmp(pObj, "cossuperdefinition") ||
-					!strcasecmp(pObj, "costemplate")
-					)
+				if(!strcasecmp(pObj, "cosdefinition") ||
+				   !strcasecmp(pObj, "cossuperdefinition") ||
+				   !strcasecmp(pObj, "costemplate"))
 				{
 					rc = 1;
 				}
