@@ -48,6 +48,13 @@ memberUidLockInit()
     return (memberuid_operation_lock = PR_NewMonitor()) != NULL;
 }
 
+void
+memberUidLockDestroy()
+{
+	PR_DestroyMonitor(memberuid_operation_lock);
+	memberuid_operation_lock = NULL;
+}
+
 void
 addDynamicGroupIfNecessary(Slapi_Entry *entry, Slapi_Mods *smods) {
     Slapi_Attr *oc_attr = NULL;
