@@ -87,6 +87,8 @@
 #include "slapi-plugin.h"
 
 #define DEFAULT_TIMEOUT 600 /* (seconds) default outbound LDAP connection */
+#define DEFAULT_FLOWCONTROL_WINDOW 1000 /* #entries sent without acknowledgment */
+#define DEFAULT_FLOWCONTROL_PAUSE 2000 /* msec of pause when #entries sent witout acknowledgment */
 #define STATUS_LEN 1024
 
 struct changecounter {
@@ -145,7 +147,15 @@ typedef struct repl5agmt {
 	int agreement_type;
 	Slapi_Counter *protocol_timeout;
 	char *maxcsn; /* agmt max csn */
+	long flowControlWindow; /* This is the maximum number of entries 
+	                         * sent without acknowledgment
+	                         */
+	long flowControlPause; /* When nb of not acknowledged entries overpass totalUpdateWindow
+	                        * This is the duration (in msec) that the RA will pause before sending the next entry
+	                        */
 	Slapi_RWLock *attr_lock; /* RW lock for all the stripped attrs */
+	int WaitForAsyncResults; /* Pass to DS_Sleep(PR_MillisecondsToInterval(WaitForAsyncResults))
+	                          * in repl5_inc_waitfor_async_results */
 } repl5agmt;
 
 /* Forward declarations */
@@ -307,7 +317,8 @@ agmt_new_from_entry(Slapi_Entry *e)
 	ra->port = slapi_entry_attr_get_int(e, type_nsds5ReplicaPort);
 	/* SSL, TLS, or other transport stuff */
 	ra->transport_flags = 0;
-	agmt_set_transportinfo_no_lock(ra, e);
+	(void) agmt_set_transportinfo_no_lock(ra, e);
+	(void) agmt_set_WaitForAsyncResults(ra, e);
 
 	/* DN to use when binding. May be empty if certain SASL auth is to be used e.g. EXTERNAL GSSAPI. */
 	ra->binddn = slapi_entry_attr_get_charptr(e, type_nsds5ReplicaBindDN);
@@ -345,6 +356,28 @@ agmt_new_from_entry(Slapi_Entry *e)
 		}
 	}
 
+	/* flow control update window. */
+	ra->flowControlWindow = DEFAULT_FLOWCONTROL_WINDOW;
+	if (slapi_entry_attr_find(e, type_nsds5ReplicaFlowControlWindow, &sattr) == 0)
+	{
+		Slapi_Value *sval;
+		if (slapi_attr_first_value(sattr, &sval) == 0)
+		{
+			ra->flowControlWindow = slapi_value_get_long(sval);
+		}
+	}
+
+	/* flow control update pause. */
+	ra->flowControlPause = DEFAULT_FLOWCONTROL_PAUSE;
+	if (slapi_entry_attr_find(e, type_nsds5ReplicaFlowControlPause, &sattr) == 0)
+	{
+		Slapi_Value *sval;
+		if (slapi_attr_first_value(sattr, &sval) == 0)
+		{
+			ra->flowControlPause = slapi_value_get_long(sval);
+		}
+	}
+
 	/* DN of entry at root of replicated area */
 	tmpstr = slapi_entry_attr_get_charptr(e, type_nsds5ReplicaRoot);
 	if (NULL != tmpstr)
@@ -1014,6 +1047,26 @@ agmt_get_pausetime(const Repl_Agmt *ra)
 	return return_value;
 }
 
+long
+agmt_get_flowcontrolwindow(const Repl_Agmt *ra)
+{
+	long return_value;
+	PR_ASSERT(NULL != ra);
+	PR_Lock(ra->lock);
+	return_value = ra->flowControlWindow;
+	PR_Unlock(ra->lock);
+	return return_value;
+}
+long
+agmt_get_flowcontrolpause(const Repl_Agmt *ra)
+{
+	long return_value;
+	PR_ASSERT(NULL != ra);
+	PR_Lock(ra->lock);
+	return_value = ra->flowControlPause;
+	PR_Unlock(ra->lock);
+	return return_value;
+}
 /*
  * Warning - reference to the long name of the agreement is returned.
  * The long name of an agreement is the DN of the agreement entry,
@@ -1677,6 +1730,27 @@ agmt_set_transportinfo_no_lock(Repl_Agmt *ra, const Slapi_Entry *e)
 	return (rc);
 }
 
+int
+agmt_set_WaitForAsyncResults(Repl_Agmt *ra, const Slapi_Entry *e)
+{
+	int wait = 0;
+	if (e) {
+		wait = slapi_entry_attr_get_int(e, type_nsds5WaitForAsyncResults);
+	}
+	if (wait <= 0) {
+		ra->WaitForAsyncResults = 100; /* 0.1 sec */
+	} else {
+		ra->WaitForAsyncResults = wait;
+	}
+	return 0;
+}
+
+int
+agmt_get_WaitForAsyncResults(Repl_Agmt *ra)
+{
+	return ra->WaitForAsyncResults;
+}
+
 int 
 agmt_set_transportinfo_from_entry(Repl_Agmt *ra, const Slapi_Entry *e) 
 {
@@ -1775,6 +1849,90 @@ agmt_set_timeout_from_entry(Repl_Agmt *ra, const Slapi_Entry *e)
 	return return_value;
 }
 
+/*
+ * Set or reset the windows of entries sent without acknowledgment.
+ * The window is used during update to determine the number of
+ * entries will be send by the replica agreement without acknowledgment from the consumer
+ *
+ * Returns 0 if window set, or -1 if an error occurred.
+ */
+int
+agmt_set_flowcontrolwindow_from_entry(Repl_Agmt *ra, const Slapi_Entry *e)
+{
+	Slapi_Attr *sattr = NULL;
+	int return_value = -1;
+
+	PR_ASSERT(NULL != ra);
+	PR_Lock(ra->lock);
+	if (ra->stop_in_progress)
+	{
+		PR_Unlock(ra->lock);
+		return return_value;
+	}
+
+	slapi_entry_attr_find(e, type_nsds5ReplicaFlowControlWindow, &sattr);
+	if (NULL != sattr)
+	{
+		Slapi_Value *sval = NULL;
+		slapi_attr_first_value(sattr, &sval);
+		if (NULL != sval)
+		{
+			long tmpval = slapi_value_get_long(sval);
+			if (tmpval >= 0) {
+				ra->flowControlWindow = tmpval;
+				return_value = 0; /* success! */
+			}
+		}
+	}
+	PR_Unlock(ra->lock);
+	if (return_value == 0)
+	{
+		prot_notify_agmt_changed(ra->protocol, ra->long_name);
+	}
+	return return_value;
+}
+
+/*
+ * Set or reset the pause duration when #entries sent without acknowledgment overpass flow control window
+ *
+ * Returns 0 if pause set, or -1 if an error occurred.
+ */
+int
+agmt_set_flowcontrolpause_from_entry(Repl_Agmt *ra, const Slapi_Entry *e)
+{
+	Slapi_Attr *sattr = NULL;
+	int return_value = -1;
+
+	PR_ASSERT(NULL != ra);
+	PR_Lock(ra->lock);
+	if (ra->stop_in_progress)
+	{
+		PR_Unlock(ra->lock);
+		return return_value;
+	}
+
+	slapi_entry_attr_find(e, type_nsds5ReplicaFlowControlPause, &sattr);
+	if (NULL != sattr)
+	{
+		Slapi_Value *sval = NULL;
+		slapi_attr_first_value(sattr, &sval);
+		if (NULL != sval)
+		{
+			long tmpval = slapi_value_get_long(sval);
+			if (tmpval >= 0) {
+				ra->flowControlPause = tmpval;
+				return_value = 0; /* success! */
+			}
+		}
+	}
+	PR_Unlock(ra->lock);
+	if (return_value == 0)
+	{
+		prot_notify_agmt_changed(ra->protocol, ra->long_name);
+	}
+	return return_value;
+}
+
 int
 agmt_set_timeout(Repl_Agmt *ra, long timeout)
 {
@@ -1788,6 +1946,32 @@ agmt_set_timeout(Repl_Agmt *ra, long timeout)
 
     return 0;
 }
+int
+agmt_set_flowcontrolwindow(Repl_Agmt *ra, long window)
+{
+    PR_Lock(ra->lock);
+    if (ra->stop_in_progress){
+        PR_Unlock(ra->lock);
+        return -1;
+    }
+    ra->flowControlWindow = window;
+    PR_Unlock(ra->lock);
+
+    return 0;
+}
+int
+agmt_set_flowcontrolpause(Repl_Agmt *ra, long pause)
+{
+    PR_Lock(ra->lock);
+    if (ra->stop_in_progress){
+        PR_Unlock(ra->lock);
+        return -1;
+    }
+    ra->flowControlPause = pause;
+    PR_Unlock(ra->lock);
+
+    return 0;
+}
 
 /*
  * Set or reset the busywaittime
@@ -2264,15 +2448,25 @@ agmt_set_last_update_status (Repl_Agmt *ra, int ldaprc, int replrc, const char *
 }
 
 void
-agmt_set_last_init_status (Repl_Agmt *ra, int ldaprc, int replrc, const char *message)
+agmt_set_last_init_status (Repl_Agmt *ra, int ldaprc, int replrc, int connrc, const char *message)
 {
+    char *connmsg = NULL;
+    char unknown_connrc[100] = {0};
+    
+    connmsg = conn_result2string(connrc);
+    if (connrc && (connmsg == NULL)) {
+        /* That was an unknown connection error */
+        PR_snprintf(unknown_connrc, sizeof(unknown_connrc), "Unknown connection error (%d)", connrc);
+        connmsg = unknown_connrc;
+    }
+    
 	PR_ASSERT(NULL != ra);
 	if (NULL != ra)
 	{
 		if (ldaprc != LDAP_SUCCESS)
 		{
 			char *replmsg = NULL;
-
+            
 			if ( replrc ) {
 				replmsg = protocol_response2string(replrc);
 				/* Do not mix the unknown replication error with the known ldap one */
@@ -2280,9 +2474,10 @@ agmt_set_last_init_status (Repl_Agmt *ra, int ldaprc, int replrc, const char *me
 					replmsg = NULL;
 				}
 			}
-			PR_snprintf(ra->last_init_status, STATUS_LEN, "%d %s%sLDAP error: %s%s%s",
+			PR_snprintf(ra->last_init_status, STATUS_LEN, "%d %s%sLDAP error: %s%s%s%s%s",
 				ldaprc, message?message:"",message?"":" - ",
-				slapi_err2string(ldaprc), replmsg ? " - " : "", replmsg ? replmsg : "");
+				slapi_err2string(ldaprc), replmsg ? " - " : "", replmsg ? replmsg : "",
+                    connrc ? " - " : "", connrc ? connmsg : "");
 		}
 		/* ldaprc == LDAP_SUCCESS */
 		else if (replrc != 0)
@@ -2316,11 +2511,18 @@ agmt_set_last_init_status (Repl_Agmt *ra, int ldaprc, int replrc, const char *me
 			else
 			{
 				PR_snprintf(ra->last_init_status, STATUS_LEN,
-					"%d Replication error acquiring replica: %s%s%s",
+					"%d Replication error acquiring replica: %s%s%s%s%s",
 					replrc, protocol_response2string(replrc),
-					message?" - ":"",message?message:"");
+					message?" - ":"",message?message:"",
+                    connrc?" - ":"",connrc?connmsg:"");
 			}
 		}
+        else if (connrc != CONN_OPERATION_SUCCESS) {
+            PR_snprintf(ra->last_init_status, STATUS_LEN,
+					"%d connection error: %s%s%s",
+					connrc, connmsg,
+					message?" - ":"",message?message:"");
+        }
 		else if (message != NULL) /* replrc == NSDS50_REPL_REPLICA_READY == 0 */
 		{
 			PR_snprintf(ra->last_init_status, STATUS_LEN,
@@ -2454,26 +2656,16 @@ get_agmt_status(Slapi_PBlock *pb, Slapi_Entry* e, Slapi_Entry* entryAfter,
 		slapi_entry_attr_delete(e, "nsds5replicaLastInitEnd");
 
 		/* now, add the real values (singly) */
-		if (ra->last_update_start_time == 0)
-		{
-			slapi_entry_add_string(e, "nsds5replicaLastUpdateStart", "0");
-		}
-		else
-		{
-			time_tmp = format_genTime(ra->last_update_start_time);
-			slapi_entry_add_string(e, "nsds5replicaLastUpdateStart", time_tmp);
-			slapi_ch_free((void **)&time_tmp);
-		}
-		if (ra->last_update_end_time == 0)
-		{
-			slapi_entry_add_string(e, "nsds5replicaLastUpdateEnd", "0");
-		}
-		else
-		{
-			time_tmp = format_genTime(ra->last_update_end_time);
-			slapi_entry_add_string(e, "nsds5replicaLastUpdateEnd", time_tmp);
-			slapi_ch_free((void **)&time_tmp);
-		}
+		/* In case last_update_start_time is not set, 19700101000000Z is set. */
+		time_tmp = format_genTime(ra->last_update_start_time);
+		slapi_entry_add_string(e, "nsds5replicaLastUpdateStart", time_tmp);
+		slapi_ch_free((void **)&time_tmp);
+
+		/* In case last_update_end_time is not set, 19700101000000Z is set. */
+		time_tmp = format_genTime(ra->last_update_end_time);
+		slapi_entry_add_string(e, "nsds5replicaLastUpdateEnd", time_tmp);
+		slapi_ch_free((void **)&time_tmp);
+
 		agmt_get_changecount_string (ra, changecount_string, sizeof (changecount_string) );
 		slapi_entry_add_string(e, "nsds5replicaChangesSentSinceStartup", changecount_string);
 		if (ra->last_update_status[0] == '\0')
@@ -2485,26 +2677,17 @@ get_agmt_status(Slapi_PBlock *pb, Slapi_Entry* e, Slapi_Entry* entryAfter,
 			slapi_entry_add_string(e, "nsds5replicaLastUpdateStatus", ra->last_update_status);
 		}
 		slapi_entry_add_string(e, "nsds5replicaUpdateInProgress", ra->update_in_progress ? "TRUE" : "FALSE");
-		if (ra->last_init_start_time == 0)
-		{
-			slapi_entry_add_string(e, "nsds5replicaLastInitStart", "0");
-		}
-		else
-		{
-			time_tmp = format_genTime(ra->last_init_start_time);
-			slapi_entry_add_string(e, "nsds5replicaLastInitStart", time_tmp);
-			slapi_ch_free((void **)&time_tmp);
-		}		
-		if (ra->last_init_end_time == 0)
-		{
-			slapi_entry_add_string(e, "nsds5replicaLastInitEnd", "0");
-		}
-		else
-		{
-			time_tmp = format_genTime(ra->last_init_end_time);
-			slapi_entry_add_string(e, "nsds5replicaLastInitEnd", time_tmp);
-			slapi_ch_free((void **)&time_tmp);
-		}		
+
+		/* In case last_init_start_time is not set, 19700101000000Z is set. */
+		time_tmp = format_genTime(ra->last_init_start_time);
+		slapi_entry_add_string(e, "nsds5replicaLastInitStart", time_tmp);
+		slapi_ch_free((void **)&time_tmp);
+
+		/* In case last_init_end_time is not set, 19700101000000Z is set. */
+		time_tmp = format_genTime(ra->last_init_end_time);
+		slapi_entry_add_string(e, "nsds5replicaLastInitEnd", time_tmp);
+		slapi_ch_free((void **)&time_tmp);
+
 		if (ra->last_init_status[0] != '\0')
 		{
 			slapi_entry_add_string(e, "nsds5replicaLastInitStatus", ra->last_init_status);
