@@ -177,6 +177,57 @@ add_new_agreement(Slapi_Entry *e)
     return rc;
 }
 
+int
+id_extended_agreement(Repl_Agmt *agmt, LDAPMod** mods, Slapi_Entry *e)
+{
+    Slapi_Attr *sattr = NULL;
+    char *val = NULL;
+    int return_value = 0;
+    int i;
+
+    slapi_entry_attr_find(e, "objectclass", &sattr);
+    if (sattr) {
+        Slapi_Value *sval = NULL;
+        const char *val = NULL;
+        for (i = slapi_attr_first_value(sattr, &sval);
+                 i >= 0; i = slapi_attr_next_value(sattr, i, &sval)) {
+            val = slapi_value_get_string(sval);
+            if ((0 == strcasecmp(val,"top")) ||
+                (0 == strcasecmp(val,"nsds5replicationAgreement"))) {
+                continue;
+            } else {
+                /* the entry has an additional objectclass, accept mods */
+                return 1;
+            }
+        }
+    }
+    /* This modification could remove an additional objectclass.
+     * In the entry we check this mod has already been applied,
+     * so check list of mods
+     */
+    for (i = 0; NULL != mods && NULL != mods[i]; i++) {
+        if (strcasecmp(mods[i]->mod_type, "objectclass")) continue;
+        if (mods[i]->mod_bvalues){
+            int j;
+            for (j = 0; mods[i]->mod_bvalues[j]; j++){
+                slapi_ch_free_string(&val);
+                val = slapi_berval_get_string_copy (mods[i]->mod_bvalues[j]);
+                if ((0 == strcasecmp(val,"top")) ||
+                    (0 == strcasecmp(val,"nsds5replicationAgreement"))) {
+                    continue;
+                } else {
+                    /* an additional objectclass was modified */
+                    return_value = 1;
+                    break;
+                }
+            }
+        }
+        break;
+    }
+    slapi_ch_free_string(&val);
+    return return_value;
+}
+
 static int
 agmtlist_add_callback(Slapi_PBlock *pb, Slapi_Entry *e, Slapi_Entry *entryAfter,
 	int *returncode, char *returntext, void *arg)
@@ -329,6 +380,32 @@ agmtlist_modify_callback(Slapi_PBlock *pb, Slapi_Entry *entryBefore, Slapi_Entry
                 rc = SLAPI_DSE_CALLBACK_ERROR;
             }
 		}
+		else if (slapi_attr_types_equivalent(mods[i]->mod_type,
+					type_nsds5ReplicaFlowControlWindow))
+		{
+			/* New replica timeout */
+			if (agmt_set_flowcontrolwindow_from_entry(agmt, e) != 0)
+			{
+				slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name, "agmtlist_modify_callback: " 
+						"failed to update the flow control window for agreement %s\n",
+						agmt_get_long_name(agmt));	
+				*returncode = LDAP_OPERATIONS_ERROR;
+				rc = SLAPI_DSE_CALLBACK_ERROR;
+			}
+		}
+		else if (slapi_attr_types_equivalent(mods[i]->mod_type,
+					type_nsds5ReplicaFlowControlPause))
+		{
+			/* New replica timeout */
+			if (agmt_set_flowcontrolpause_from_entry(agmt, e) != 0)
+			{
+				slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name, "agmtlist_modify_callback: " 
+						"failed to update the flow control pause for agreement %s\n",
+						agmt_get_long_name(agmt));	
+				*returncode = LDAP_OPERATIONS_ERROR;
+				rc = SLAPI_DSE_CALLBACK_ERROR;
+			}
+		}
 		else if (slapi_attr_types_equivalent(mods[i]->mod_type,
 					type_nsds5ReplicaBusyWaitTime))
 		{
@@ -498,8 +575,7 @@ agmtlist_modify_callback(Slapi_PBlock *pb, Slapi_Entry *entryBefore, Slapi_Entry
 			repl5_set_debug_timeout(val);
 			slapi_ch_free_string(&val);
 		}
-        else if (strcasecmp (mods[i]->mod_type, "modifytimestamp") == 0 ||
-                 strcasecmp (mods[i]->mod_type, "modifiersname") == 0 ||
+        else if (slapi_attr_is_last_mod(mods[i]->mod_type) ||
                  strcasecmp (mods[i]->mod_type, "description") == 0)
         {
             /* ignore modifier's name and timestamp attributes and the description. */
@@ -523,7 +599,8 @@ agmtlist_modify_callback(Slapi_PBlock *pb, Slapi_Entry *entryBefore, Slapi_Entry
                 rc = SLAPI_DSE_CALLBACK_ERROR;
             }
         }
-        else if (slapi_attr_types_equivalent(mods[i]->mod_type, type_replicaProtocolTimeout)){
+        else if (slapi_attr_types_equivalent(mods[i]->mod_type, type_replicaProtocolTimeout))
+        {
             if (mods[i]->mod_op & LDAP_MOD_DELETE)
             {
                 agmt_set_protocol_timeout(agmt, 0);
@@ -549,7 +626,16 @@ agmtlist_modify_callback(Slapi_PBlock *pb, Slapi_Entry *entryBefore, Slapi_Entry
                 agmt_set_protocol_timeout(agmt, ptimeout);
             }
         }
-        else if (0 == windows_handle_modify_agreement(agmt, mods[i]->mod_type, e))
+        else if (slapi_attr_types_equivalent(mods[i]->mod_type, type_nsds5WaitForAsyncResults))
+        {
+            if (mods[i]->mod_op & LDAP_MOD_DELETE) {
+                (void) agmt_set_WaitForAsyncResults(agmt, NULL);
+            } else {
+                (void) agmt_set_WaitForAsyncResults(agmt, e);
+            }
+        }
+        else if ((0 == windows_handle_modify_agreement(agmt, mods[i]->mod_type, e)) &&
+                 (0 == id_extended_agreement(agmt, mods, e)))
         {
             slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name, "agmtlist_modify_callback: " 
                             "modification of %s attribute is not allowed\n", mods[i]->mod_type);
@@ -567,7 +653,7 @@ agmtlist_modify_callback(Slapi_PBlock *pb, Slapi_Entry *entryBefore, Slapi_Entry
     {
         if (agmt_initialize_replica(agmt) != 0) {
             /* The suffix/repl agmt is disabled */
-            agmt_set_last_init_status(agmt, 0, NSDS50_REPL_DISABLED, NULL);
+            agmt_set_last_init_status(agmt, 0, NSDS50_REPL_DISABLED, 0, NULL);
             if(agmt_is_enabled(agmt)){
                 PR_snprintf(returntext, SLAPI_DSE_RETURNTEXT_SIZE, "Suffix is disabled");
             } else {
