@@ -82,6 +82,7 @@ typedef struct callback_data
 	int stop_result_thread; /* Flag used to tell the result thread to exit */
 	int last_message_id_sent;
 	int last_message_id_received;
+	int flowcontrol_detection;
 } callback_data;
 
 /* 
@@ -368,7 +369,7 @@ repl5_tot_run(Private_Repl_Protocol *prp)
 		int optype, ldaprc;
 		conn_get_error(prp->conn, &optype, &ldaprc);
 		agmt_set_last_init_status(prp->agmt, ldaprc,
-				  prp->last_acquire_response_code, NULL);
+				  prp->last_acquire_response_code, 0, NULL);
         goto done;
     }
 	else if (prp->terminate)
@@ -380,7 +381,7 @@ repl5_tot_run(Private_Repl_Protocol *prp)
 	hostname = agmt_get_hostname(prp->agmt);
 	portnum = agmt_get_port(prp->agmt);
 
-    agmt_set_last_init_status(prp->agmt, 0, 0, "Total schema update in progress");
+    agmt_set_last_init_status(prp->agmt, 0, 0, 0, "Total schema update in progress");
 	remote_schema_csn = agmt_get_consumer_schema_csn ( prp->agmt );
 	rc = conn_push_schema(prp->conn, &remote_schema_csn);
 
@@ -395,11 +396,11 @@ repl5_tot_run(Private_Repl_Protocol *prp)
 						"total update session.\n",
 						hostname, portnum);
 		/* But keep going */
-		agmt_set_last_init_status(prp->agmt, 0, rc, "Total schema update failed");
+		agmt_set_last_init_status(prp->agmt, 0, rc, 0, "Total schema update failed");
 	}
 	else
 	{
-		agmt_set_last_init_status(prp->agmt, 0, 0, "Total schema update succeeded");
+		agmt_set_last_init_status(prp->agmt, 0, 0, 0, "Total schema update succeeded");
 	}
 
     /* ONREPL - big assumption here is that entries a returned in the id order
@@ -408,7 +409,7 @@ repl5_tot_run(Private_Repl_Protocol *prp)
        properly updated because bulk import at the moment skips orphand entries. */
 	/* XXXggood above assumption may not be valid if orphaned entry moved???? */
 
-    agmt_set_last_init_status(prp->agmt, 0, 0, "Total update in progress");
+    agmt_set_last_init_status(prp->agmt, 0, 0, 0, "Total update in progress");
 
     slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Beginning total update of replica "
 		"\"%s\".\n", agmt_get_long_name(prp->agmt));
@@ -428,13 +429,19 @@ repl5_tot_run(Private_Repl_Protocol *prp)
                                   LDAP_SCOPE_SUBTREE, "(|(objectclass=ldapsubentry)(objectclass=nstombstone)(nsuniqueid=*))", NULL, 0, ctrls, NULL, 
                                   repl_get_plugin_identity (PLUGIN_MULTIMASTER_REPLICATION), 0);
 
-    cb_data.prp = prp;
-    cb_data.rc = 0;
+	cb_data.prp = prp;
+	cb_data.rc = 0;
 	cb_data.num_entries = 0UL;
 	cb_data.sleep_on_busy = 0UL;
 	cb_data.last_busy = current_time ();
+	cb_data.flowcontrol_detection = 0;
 	cb_data.lock = PR_NewLock();
 
+	/* This allows during perform_operation to check the callback data
+	 * especially to do flow contol on delta send msgid / recv msgid
+	 */
+	conn_set_tot_update_cb(prp->conn, (void *) &cb_data);
+    
 	/* Before we get started on sending entries to the replica, we need to 
 	 * setup things for async propagation: 
 	 * 1. Create a thread that will read the LDAP results from the connection.
@@ -452,7 +459,9 @@ repl5_tot_run(Private_Repl_Protocol *prp)
 	}
 
     /* this search get all the entries from the replicated area including tombstones
-       and referrals */
+       and referrals 
+       Note that cb_data.rc contains values from ConnResult
+     */
     slapi_search_internal_callback_pb (pb, &cb_data /* callback data */,
                                        get_result /* result callback */,
                                        send_entry /* entry callback */,
@@ -467,7 +476,7 @@ repl5_tot_run(Private_Repl_Protocol *prp)
 
 	if (!prp->repl50consumer) 
 	{
-		if(cb_data.rc == LDAP_SUCCESS){ /* no need to wait if we already failed */
+		if(cb_data.rc == CONN_OPERATION_SUCCESS){ /* no need to wait if we already failed */
 			repl5_tot_waitfor_async_results(&cb_data);
 		}
 		rc = repl5_tot_destroy_async_result_thread(&cb_data);
@@ -491,21 +500,32 @@ repl5_tot_run(Private_Repl_Protocol *prp)
 	agmt_update_done(prp->agmt, 1);
 	release_replica(prp);
 	
-	if (rc != LDAP_SUCCESS)
+	if (rc != CONN_OPERATION_SUCCESS)
 	{
 		slapi_log_error (SLAPI_LOG_FATAL, repl_plugin_name, "Total update failed for replica \"%s\", "
 				         "error (%d)\n", agmt_get_long_name(prp->agmt), rc);
-		agmt_set_last_init_status(prp->agmt, rc, 0, "Total update aborted");
+		agmt_set_last_init_status(prp->agmt, 0, 0, rc, "Total update aborted");
 	} else {
 		slapi_log_error (SLAPI_LOG_FATAL, repl_plugin_name, "Finished total update of replica "
 		                 "\"%s\". Sent %lu entries.\n",
 		                 agmt_get_long_name(prp->agmt), cb_data.num_entries);
-		agmt_set_last_init_status(prp->agmt, 0, 0, "Total update succeeded");
+		agmt_set_last_init_status(prp->agmt, 0, 0, 0, "Total update succeeded");
 	}
 
 done:
 	slapi_sdn_free(&area_sdn);
 	slapi_ch_free_string(&hostname);
+	if (cb_data.flowcontrol_detection > 1)
+	{
+		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+			"%s: Total update flow control triggered %d times\n"
+			"You may increase %s and/or decrease %s in the replica agreement configuration\n",
+			agmt_get_long_name(prp->agmt),
+			cb_data.flowcontrol_detection,
+			type_nsds5ReplicaFlowControlPause,
+			type_nsds5ReplicaFlowControlWindow);
+	}
+	conn_set_tot_update_cb(prp->conn, NULL);
 	if (cb_data.lock) 
 	{
 		PR_DestroyLock(cb_data.lock);
@@ -645,6 +665,37 @@ void get_result (int rc, void *cb_data)
     ((callback_data*)cb_data)->rc = rc;
 }
 
+/* Call must hold the connection lock */
+int
+repl5_tot_last_rcv_msgid(Repl_Connection *conn) 
+{
+    struct callback_data *cb_data;
+    
+    conn_get_tot_update_cb_nolock(conn, (void **) &cb_data);
+    if (cb_data == NULL) {
+        return -1;
+    } else {
+        return cb_data->last_message_id_received;
+    }
+}
+
+/* Increase the flowcontrol counter
+ * Call must hold the connection lock
+ */
+int
+repl5_tot_flowcontrol_detection(Repl_Connection *conn, int increment) 
+{
+    struct callback_data *cb_data;
+    
+    conn_get_tot_update_cb_nolock(conn, (void **) &cb_data);
+    if (cb_data == NULL) {
+        return -1;
+    } else {
+        cb_data->flowcontrol_detection += increment;
+        return cb_data->flowcontrol_detection;
+    }
+}
+
 static 
 int send_entry (Slapi_Entry *e, void *cb_data)
 {
@@ -776,12 +827,14 @@ int send_entry (Slapi_Entry *e, void *cb_data)
 	if (CONN_NOT_CONNECTED == rc) {
 		((callback_data*)cb_data)->rc = -2;
 		retval = -1;
-	} else if (CONN_OPERATION_SUCCESS == rc) {
-		retval = 0;
 	} else {
-		((callback_data*)cb_data)->rc = rc;
-		retval = -1;
-	}
+        ((callback_data*) cb_data)->rc = rc;
+        if (CONN_OPERATION_SUCCESS == rc) {
+            retval = 0;
+        } else {
+            retval = -1;
+        }
+    }
 error:
 	return retval;
 }
