@@ -595,7 +595,7 @@ windows_LDAPMessage2Entry(Slapi_Entry *e, Repl_Connection *conn,
 				char *dupa = slapi_ch_strdup(a);
 				char *newa = NULL; /* dup of 'a' with next range */
 				char *p, *wp, *pp; /* work pointers */
-				char *iter;
+				char *iter = NULL;
 				int high = 0;
 				int sizea = strlen(a) + 2;
 				/* handling subtype(s) */
@@ -814,10 +814,6 @@ next:
 		slapi_ch_free_string(&filter_copy);
 		ldap_controls_free(serverctrls_copy);
 		serverctrls_copy = NULL;
-
-		/* clear it here in case the search fails and
-		   we are left with a bogus old entry */
-		windows_private_set_raw_entry(conn->agmt, NULL);
 		conn->last_ldap_error = ldap_rc;
 		if (NULL != res)
 		{
