@@ -101,6 +101,12 @@ static lenstr *make_changes_string(LDAPMod **ldm, const char **includeattrs)
 		continue;
 	    }
 	}
+	if (SLAPD_UNHASHED_PW_NOLOG == slapi_config_get_unhashed_pw_switch()) {
+		if (0 == strcasecmp(ldm[ i ]->mod_type, PSEUDO_ATTR_UNHASHEDUSERPASSWORD)) {
+			/* If nsslapd-unhashed-pw-switch == nolog, skip writing it to cl. */
+			continue;
+		}
+	}
 	switch ( ldm[ i ]->mod_op  & ~LDAP_MOD_BVALUES ) {
 	case LDAP_MOD_ADD:
 	    addlenstr( l, "add: " );
@@ -231,7 +237,7 @@ write_replog_db(
             if ( entry == NULL ) {
                 slapi_pblock_get( pb, SLAPI_ENTRY_PRE_OP, &entry );
             }
-
+            if ( entry == NULL ) continue;
             uniqueId = slapi_entry_get_uniqueid( entry );
 
             slapi_log_error( SLAPI_LOG_PLUGIN, RETROCL_PLUGIN_NAME,
@@ -597,6 +603,10 @@ int retrocl_postob (Slapi_PBlock *pb, int optype)
      */
    
     (void)slapi_pblock_get( pb, SLAPI_BACKEND, &be );
+    if (be == NULL) {
+        LDAPDebug0Args(LDAP_DEBUG_ANY, "retrocl_postob: backend is not set\n");
+        return SLAPI_PLUGIN_FAILURE;
+    }
     
     if (slapi_be_logchanges(be) == 0) {
         LDAPDebug0Args(LDAP_DEBUG_TRACE,"not applying change if not logging\n");
