@@ -136,7 +136,7 @@ ldbm_back_delete( Slapi_PBlock *pb )
 	if (pb->pb_conn)
 	{
 		slapi_log_error (SLAPI_LOG_TRACE, "ldbm_back_delete", "enter conn=%" NSPRIu64 " op=%d\n",
-				(long long unsigned int)pb->pb_conn->c_connid, operation->o_opid);
+				pb->pb_conn->c_connid, operation->o_opid);
 	}
 
 	if ((NULL == addr) || (NULL == sdnp))
@@ -594,7 +594,8 @@ ldbm_back_delete( Slapi_PBlock *pb )
 				char *tombstone_dn;
 				Slapi_Value *tomb_value;
 
-				if (slapi_is_special_rdn(edn, RDN_IS_TOMBSTONE)) {
+				if (slapi_entry_attr_hasvalue(e->ep_entry, SLAPI_ATTR_OBJECTCLASS, SLAPI_ATTR_VALUE_TOMBSTONE) &&
+				    slapi_is_special_rdn(edn, RDN_IS_TOMBSTONE)) {
 					slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_delete",
 					                "conn=%lu op=%d Turning a tombstone into a tombstone! \"%s\"; e: 0x%p, cache_state: 0x%x, refcnt: %d\n", 
 					                conn_id, op_id, edn, e, e->ep_state, e->ep_refcnt);
@@ -602,7 +603,21 @@ ldbm_back_delete( Slapi_PBlock *pb )
 					retval = -1;
 					goto error_return;
 				}
-				tombstone_dn = compute_entry_tombstone_dn(edn, childuniqueid);
+				if (!childuniqueid) {
+					slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_delete",
+					                "conn=%lu op=%d No nsUniqueId in the entry \"%s\"; e: 0x%p, cache_state: 0x%x, refcnt: %d\n", 
+					                conn_id, op_id, edn, e, e->ep_state, e->ep_refcnt);
+					ldap_result_code= LDAP_OPERATIONS_ERROR;
+					retval = -1;
+					goto error_return;
+				}
+				if ((0 == PL_strncmp(edn + sizeof(SLAPI_ATTR_UNIQUEID), childuniqueid, strlen(childuniqueid))) &&
+				    (*(edn + SLAPI_ATTR_UNIQUEID_LENGTH + slapi_uniqueIDSize() + 1/*=*/) == ',')) {
+					/* The DN already starts with "nsuniqueid=...," */
+					tombstone_dn = slapi_ch_strdup(edn);
+				} else {
+					tombstone_dn = compute_entry_tombstone_dn(edn, childuniqueid);
+				}
 
 				slapi_sdn_set_ndn_byval(&nscpEntrySDN, slapi_sdn_get_ndn(slapi_entry_get_sdn(e->ep_entry)));
 
@@ -1261,6 +1276,16 @@ ldbm_back_delete( Slapi_PBlock *pb )
 
 	/* delete from cache and clean up */
 	if (e) {
+		if(!create_tombstone_entry){
+			struct backentry *old_e = e;
+			e = cache_find_id(&inst->inst_cache,e->ep_id);
+			if(e != old_e){
+				cache_unlock_entry(&inst->inst_cache, old_e);
+				CACHE_RETURN(&inst->inst_cache, &old_e);
+			} else {
+				CACHE_RETURN(&inst->inst_cache, &e);
+			}
+		}
 		if (cache_is_in_cache(&inst->inst_cache, e)) {
 			ep_id = e->ep_id; /* Otherwise, e might have been freed. */
 			CACHE_REMOVE(&inst->inst_cache, e);
@@ -1490,7 +1515,7 @@ diskfull_return:
 	if (pb->pb_conn)
 	{
 		slapi_log_error (SLAPI_LOG_TRACE, "ldbm_back_delete", "leave conn=%" NSPRIu64 " op=%d\n",
-				(long long unsigned int)pb->pb_conn->c_connid, operation->o_opid);
+				pb->pb_conn->c_connid, operation->o_opid);
 	}
 
 	return rc;
