@@ -76,7 +76,7 @@
 static void log_bind_access(
     Slapi_PBlock *pb, 
     const char* dn, 
-    int method, 
+    ber_tag_t method, 
     int version,
     const char *saslmech,
     const char *msg
@@ -777,7 +777,8 @@ do_bind( Slapi_PBlock *pb )
                          * was in be_bind.  Since be_bind returned SLAPI_BIND_SUCCESS,
                          * the entry is in the DS.  So, we need to retrieve it once more.
                          */
-                        if (!bind_target_entry) {
+                        if (!slapi_be_is_flag_set(be, SLAPI_BE_FLAG_REMOTE_DATA) && 
+                            !bind_target_entry) {
                             bind_target_entry = get_entry(pb, slapi_sdn_get_ndn(sdn));
                             if (bind_target_entry) {
                                 myrc = slapi_check_account_lock(pb, bind_target_entry,
@@ -889,7 +890,7 @@ static void
 log_bind_access (
     Slapi_PBlock *pb, 
     const char* dn, 
-    int method, 
+    ber_tag_t method, 
     int version,
     const char *saslmech,
     const char *msg
@@ -899,25 +900,25 @@ log_bind_access (
         slapi_log_access( LDAP_DEBUG_STATS, 
                           "conn=%" NSPRIu64 " op=%d BIND dn=\"%s\" "
                           "method=sasl version=%d mech=%s, %s\n",
-                          (long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid, dn,
+                          pb->pb_conn->c_connid, pb->pb_op->o_opid, dn,
                           version, saslmech, msg );
     } else if (method == LDAP_AUTH_SASL && saslmech) {
         slapi_log_access( LDAP_DEBUG_STATS, 
                           "conn=%" NSPRIu64 " op=%d BIND dn=\"%s\" "
                           "method=sasl version=%d mech=%s\n",
-                          (long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid, dn,
+                          pb->pb_conn->c_connid, pb->pb_op->o_opid, dn,
                           version, saslmech );
     } else if (msg) {
         slapi_log_access( LDAP_DEBUG_STATS, 
                           "conn=%" NSPRIu64 " op=%d BIND dn=\"%s\" "
-                          "method=%d version=%d, %s\n",
-                          (long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid, dn,
+                          "method=%ld version=%d, %s\n",
+                          pb->pb_conn->c_connid, pb->pb_op->o_opid, dn,
                           method, version, msg );
     } else {
         slapi_log_access( LDAP_DEBUG_STATS, 
                           "conn=%" NSPRIu64 " op=%d BIND dn=\"%s\" "
-                          "method=%d version=%d\n",
-                          (long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid, dn,
+                          "method=%ld version=%d\n",
+                          pb->pb_conn->c_connid, pb->pb_op->o_opid, dn,
                           method, version );
     }
 }
