@@ -157,7 +157,7 @@ static struct dse_node *dse_find_node( struct dse* pdse, const Slapi_DN *dn );
 static int dse_modify_plugin(Slapi_Entry *pre_entry, Slapi_Entry *post_entry, char *returntext);
 static int dse_add_plugin(Slapi_Entry *entry, char *returntext);
 static int dse_delete_plugin(Slapi_Entry *entry, char *returntext);
-static void dse_post_modify_plugin(Slapi_Entry *entryBefore, Slapi_Entry *entryAfter, LDAPMod **mods);
+static int dse_pre_modify_plugin(Slapi_Entry *entryBefore, Slapi_Entry *entryAfter, LDAPMod **mods);
 
 /*
   richm: In almost all modes e.g. db2ldif, ldif2db, etc. we do not need/want
@@ -1499,7 +1499,7 @@ dse_delete_entry(struct dse* pdse, Slapi_PBlock *pb, const Slapi_Entry *e)
 int
 dse_bind( Slapi_PBlock *pb ) /* JCM There should only be one exit point from this function! */
 {
-	int	method; /* The bind method */
+	ber_tag_t method; /* The bind method */
 	struct berval *cred; /* The bind credentials */
 	Slapi_Value **bvals; 
     struct dse* pdse;
@@ -1827,6 +1827,8 @@ dse_modify(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
     int retval = -1;
     int need_be_postop = 0;
     int plugin_started = 0;
+    int internal_op = 0;
+    PRBool global_lock_owned = PR_FALSE;
 
     PR_ASSERT(pb);
     if (slapi_pblock_get( pb, SLAPI_PLUGIN_PRIVATE, &pdse ) < 0 ||
@@ -1843,6 +1845,8 @@ dse_modify(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
         return retval;
     }
 
+    internal_op = operation_is_flag_set(pb->pb_op, OP_FLAG_INTERNAL);
+
     /* Find the entry we are about to modify. */
     ec = dse_get_entry_copy(pdse, sdn, DSE_USE_LOCK);
     if ( ec == NULL ) {
@@ -1868,6 +1872,12 @@ dse_modify(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
     /* Modify a copy of the entry*/
     ecc = slapi_entry_dup( ec );
     err = entry_apply_mods( ecc, mods );
+    
+    /* Possibly acquire the global backend lock */
+    if (global_backend_lock_requested()) {
+        global_backend_lock_lock();
+        global_lock_owned = PR_TRUE;
+    }
 
     /* XXXmcs: should we expand objectclass values here?? */
     /* give the dse callbacks the first crack at the modify */
@@ -1913,13 +1923,41 @@ dse_modify(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
             }
         } else {
             /*
-             * Check if we are enabling/disabling a plugin
+             * If we are using dynamic plugins, and we are modifying a plugin
+             * we need to do some additional checks.  First, check if we are
+             * enabling/disabling a plugin.  Then make sure the plugin still
+             * starts after applying the plugin changes.
              */
-            if((plugin_started = dse_modify_plugin(ec, ecc, returntext)) == -1){
-                returncode = LDAP_UNWILLING_TO_PERFORM;
-                rc = SLAPI_DSE_CALLBACK_ERROR;
-            } else {
-                rc = SLAPI_DSE_CALLBACK_OK;
+        	rc = SLAPI_DSE_CALLBACK_OK;
+            if(config_get_dynamic_plugins() &&
+               slapi_entry_attr_hasvalue(ec, SLAPI_ATTR_OBJECTCLASS, "nsSlapdPlugin") )
+            {
+                if((plugin_started = dse_modify_plugin(ec, ecc, returntext)) == -1){
+                    returncode = LDAP_UNWILLING_TO_PERFORM;
+                    rc = SLAPI_DSE_CALLBACK_ERROR;
+                    retval = -1;
+                    goto done;
+                }
+                /*
+                 * If this is not a internal operation, make sure the plugin
+                 * can be restarted.
+                 */
+                if(!internal_op){
+                    if(dse_pre_modify_plugin(ec, ecc, mods)){
+                        char *errtext;
+                        slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &errtext);
+                        if (errtext) {
+                            PL_strncpyz(returntext,
+                                        "Failed to apply plugin config change, "
+                                        "check the errors log for more info.",
+                                        sizeof(returntext));
+                        }
+                        returncode = LDAP_UNWILLING_TO_PERFORM;
+                        rc = SLAPI_DSE_CALLBACK_ERROR;
+                        retval = -1;
+                        goto done;
+                    }
+                }
             }
         }
     }
@@ -2038,41 +2076,22 @@ dse_modify(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
             }
         }
     }
-    /*
-     * Perform postop plugin configuration changes
-     */
-    if(returncode == LDAP_SUCCESS){
-        dse_post_modify_plugin(ec, ecc, mods);
-    } else if(plugin_started){
-        if(plugin_started == 1){
-        	/* the op failed, turn the plugin off */
-            plugin_delete(ecc, returntext, 0 /* not locked */);
-        } else if (plugin_started == 2){
-            /*
-             * This probably can't happen, but...
-             * the op failed, turn the plugin back on.
-             */
-            plugin_add(ecc, returntext, 0 /* not locked */);
-        }
+    if (global_lock_owned) {
+        global_backend_lock_unlock();
     }
-
     slapi_send_ldap_result( pb, returncode, NULL, returntext[0] ? returntext : NULL, 0, NULL );
 
     return dse_modify_return(retval, ec, ecc);
 }
 
-static void
-dse_post_modify_plugin(Slapi_Entry *entryBefore, Slapi_Entry *entryAfter, LDAPMod **mods)
+static int
+dse_pre_modify_plugin(Slapi_Entry *entryBefore, Slapi_Entry *entryAfter, LDAPMod **mods)
 {
     char *enabled = NULL;
+    int restart_plugin = 1;
+    int rc = 0;
     int i;
 
-    if (!slapi_entry_attr_hasvalue(entryBefore, SLAPI_ATTR_OBJECTCLASS, "nsSlapdPlugin") ||
-            !config_get_dynamic_plugins() ){
-        /* not a plugin, or we aren't applying updates dynamically - just move on */
-        return;
-    }
-
     /*
      * Only check the mods if the plugin is enabled - no need to restart a plugin if it's not running.
      */
@@ -2080,35 +2099,23 @@ dse_post_modify_plugin(Slapi_Entry *entryBefore, Slapi_Entry *entryAfter, LDAPMo
          !strcasecmp(enabled, "on"))
     {
         for(i = 0; mods && mods[i]; i++){
-            /* Check if we are modifying a plugin's config, if so restart the plugin */
-            if (strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_PATH) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_INITFN) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_TYPE) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_DEPENDS_ON_TYPE) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_DEPENDS_ON_NAMED) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_SCHEMA_CHECK) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_BE_TXN) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_TARGET_SUBTREE) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_EXCLUDE_TARGET_SUBTREE) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_BIND_SUBTREE) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_EXCLUDE_BIND_SUBTREE) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_LOAD_NOW) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_LOAD_GLOBAL) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_PRECEDENCE) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_LOG_ACCESS) == 0 ||
-                strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_LOG_AUDIT) == 0 )
-            {
-                /* for all other plugin config changes, restart the plugin */
-                if(plugin_restart(entryBefore, entryAfter) != LDAP_SUCCESS){
-                    slapi_log_error(SLAPI_LOG_FATAL,"dse_post_modify_plugin", "The configuration change "
-                            "for plugin (%s) could not be applied dynamically, and will be ignored until "
-                            "the server is restarted.\n",
-                            slapi_entry_get_dn(entryBefore));
-                }
+            if (strcasecmp(mods[i]->mod_type, ATTR_PLUGIN_ENABLED) == 0){
+                /* we already stop/started the pugin - don't do it again */
+                restart_plugin = 0;
+                break;
+            }
+        }
+        if(restart_plugin){       /* for all other plugin config changes, restart the plugin */
+            if(plugin_restart(entryBefore, entryAfter) != LDAP_SUCCESS){
+                slapi_log_error(SLAPI_LOG_FATAL,"dse_pre_modify_plugin",
+                        "The configuration change for plugin (%s) could not be applied.\n",
+                        slapi_entry_get_dn(entryBefore));
+                rc = -1;
             }
         }
     }
     slapi_ch_free_string(&enabled);
+    return rc;
 }
 
 /*
@@ -2125,12 +2132,6 @@ dse_modify_plugin(Slapi_Entry *pre_entry, Slapi_Entry *post_entry, char *returnt
 {
     int rc = LDAP_SUCCESS;
 
-    if (!slapi_entry_attr_hasvalue(post_entry, SLAPI_ATTR_OBJECTCLASS, "nsSlapdPlugin") ||
-        !config_get_dynamic_plugins() )
-    {
-        return rc;
-    }
-
     if( slapi_entry_attr_hasvalue(pre_entry, "nsslapd-pluginEnabled", "on") &&
         slapi_entry_attr_hasvalue(post_entry, "nsslapd-pluginEnabled", "off") )
     {
@@ -2232,6 +2233,7 @@ dse_add(Slapi_PBlock *pb) /* JCM There should only be one exit point from this f
     Slapi_DN *sdn = NULL;
     Slapi_DN parent;
     int need_be_postop = 0;
+    PRBool global_lock_owned = PR_FALSE;
 
     /*
      * Get the database, the dn and the entry to add
@@ -2353,6 +2355,12 @@ dse_add(Slapi_PBlock *pb) /* JCM There should only be one exit point from this f
         goto done;
     }
 
+    /* Possibly acquire the global backend lock */
+    if (global_backend_lock_requested()) {
+        global_backend_lock_lock();
+        global_lock_owned = PR_TRUE;
+    }
+
     if(dse_call_callback(pdse, pb, SLAPI_OPERATION_ADD, DSE_FLAG_PREOP, e,
                          NULL, &returncode, returntext)!=SLAPI_DSE_CALLBACK_OK) {
         if (!returncode) {
@@ -2432,9 +2440,9 @@ dse_add(Slapi_PBlock *pb) /* JCM There should only be one exit point from this f
             slapi_pblock_get(pb, SLAPI_RESULT_CODE, &returncode);
         }
     }
-
-    /* entry has been freed, so make sure no one tries to use it later */
-    slapi_pblock_set(pb, SLAPI_ADD_ENTRY, NULL);
+    if (global_lock_owned) {
+        global_backend_lock_unlock();
+    }
     slapi_send_ldap_result(pb, returncode, NULL, returntext[0] ? returntext : NULL, 0, NULL );
     return dse_add_return(rc, e);
 }
@@ -2466,6 +2474,7 @@ dse_delete(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
     Slapi_Entry *ec = NULL; /* copy of entry to delete */
     Slapi_Entry *orig_entry = NULL;
     int need_be_postop = 0;
+    PRBool global_lock_owned = PR_FALSE;
 
     /*
      * Get the database and the dn
@@ -2510,6 +2519,12 @@ dse_delete(Slapi_PBlock *pb) /* JCM There should only be one exit point from thi
         goto done;
     }
 
+    /* Possibly acquire the global backend lock */
+    if (global_backend_lock_requested()) {
+        global_backend_lock_lock();
+        global_lock_owned = PR_TRUE;
+    }
+
     if(dse_call_callback(pdse, pb, SLAPI_OPERATION_DELETE, DSE_FLAG_PREOP, ec, NULL, &returncode,returntext)==SLAPI_DSE_CALLBACK_OK) {
         slapi_pblock_set(pb, SLAPI_DELETE_BEPREOP_ENTRY, ec);
         slapi_pblock_set(pb, SLAPI_RESULT_CODE, &returncode);
@@ -2562,6 +2577,9 @@ done:
             slapi_pblock_get(pb, SLAPI_RESULT_CODE, &returncode);
         }
     }
+    if (global_lock_owned) {
+        global_backend_lock_unlock();
+    }
     if (returncode && !returntext[0]) {
         char *ldap_result_message = NULL;
         slapi_pblock_get(pb, SLAPI_PB_RESULT_TEXT, &ldap_result_message);
@@ -2690,7 +2708,10 @@ slapi_config_register_callback_plugin(int operation,
             Slapi_DN dn;
 
             slapi_sdn_init_dn_byref(&dn,base);
-            rc = (NULL != dse_register_callback(pdse, operation, flags, &dn, scope, filter, fn, fn_arg, pb ? pb->pb_plugin: NULL));
+            /* if a pblock was passed, this is a plugin, so set the f_arg as the plugin */
+            rc = (NULL != dse_register_callback(pdse, operation, flags, &dn, scope, filter, fn,
+                                                pb ? (void*)pb->pb_plugin : fn_arg,
+                                                pb ? pb->pb_plugin: NULL));
             slapi_sdn_done(&dn);
         }
     }
