@@ -423,6 +423,7 @@ static void resolve_attribute_state_present_to_deleted(Slapi_Entry *e, Slapi_Att
 static void resolve_attribute_state_to_present_or_deleted(Slapi_Entry *e, Slapi_Attr *a, Slapi_Value **valuestoupdate, int attribute_state);
 static int entry_add_present_values_wsi_single_valued(Slapi_Entry *e, const char *type, struct berval **bervals, const CSN *csn, int urp, long flags);
 static int entry_add_present_values_wsi_multi_valued(Slapi_Entry *e, const char *type, struct berval **bervals, const CSN *csn, int urp, long flags);
+
 static int
 entry_add_present_values_wsi(Slapi_Entry *e, const char *type, struct berval **bervals, const CSN *csn, int urp, long flags)
 {
@@ -439,11 +440,11 @@ entry_add_present_values_wsi(Slapi_Entry *e, const char *type, struct berval **b
 
 	if(slapi_attr_flag_is_set(a,SLAPI_ATTR_FLAG_SINGLE))
 	{
-		retVal = entry_add_present_values_wsi_single_valued( e, type, bervals, csn, urp, 0 );
+		retVal = entry_add_present_values_wsi_single_valued( e, a->a_type, bervals, csn, urp, 0 );
 	}
 	else
 	{
-		retVal = entry_add_present_values_wsi_multi_valued( e, type, bervals, csn, urp, 0 );
+		retVal = entry_add_present_values_wsi_multi_valued( e, a->a_type, bervals, csn, urp, 0 );
 	}
 	return retVal;
 }
@@ -679,9 +680,13 @@ entry_delete_present_values_wsi(Slapi_Entry *e, const char *type, struct berval
 			}
 			retVal = LDAP_NO_SUCH_ATTRIBUTE;
 		}
-		if ((LDAP_MOD_REPLACE == mod_op) && replacevals && replacevals[0])
+		if ((LDAP_MOD_REPLACE == mod_op))
 		{
-			/* Create a new attribute and set the adcsn */
+			/* Create a new attribute and set the adcsn 
+                         * We need to do this always to set the deletion csn
+                         * The attribute would otherwise be created in the 
+                         * entry_add_.. function
+                         */
 			Slapi_Attr *a = slapi_attr_new();
 			slapi_attr_init(a, type);
 			attr_set_deletion_csn(a,csn); 
