@@ -282,14 +282,14 @@ do_extended( Slapi_PBlock *pb )
 		LDAPDebug( LDAP_DEBUG_ARGS, "do_extended: oid (%s)\n", extoid, 0, 0 );
 
 		slapi_log_access( LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d EXT oid=\"%s\"\n",
-				(long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid, extoid );
+				pb->pb_conn->c_connid, pb->pb_op->o_opid, extoid );
 	} else {
 		LDAPDebug( LDAP_DEBUG_ARGS, "do_extended: oid (%s-%s)\n",
 				extoid, name, 0 );
 
 		slapi_log_access( LDAP_DEBUG_STATS,
 			"conn=%" NSPRIu64 " op=%d EXT oid=\"%s\" name=\"%s\"\n",
-			(long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid, extoid, name );
+			pb->pb_conn->c_connid, pb->pb_op->o_opid, extoid, name );
 	}
 
 	/* during a bulk import, only BULK_IMPORT_DONE is allowed! 
@@ -361,6 +361,8 @@ do_extended( Slapi_PBlock *pb )
 
 	slapi_pblock_set( pb, SLAPI_EXT_OP_REQ_OID, extoid );
 	slapi_pblock_set( pb, SLAPI_EXT_OP_REQ_VALUE, &extval );
+	slapi_pblock_set( pb, SLAPI_REQUESTOR_ISROOT, &pb->pb_op->o_isroot);
+	
 	rc = plugin_call_exop_plugins( pb, extoid );
 
 	if ( SLAPI_PLUGIN_EXTENDED_SENT_RESULT != rc ) {
