@@ -493,7 +493,7 @@ op_shared_rename(Slapi_PBlock *pb, int passin_args)
 		{
 			slapi_log_access(LDAP_DEBUG_STATS,
 					 "conn=%" NSPRIu64 " op=%d MODRDN dn=\"%s\" newrdn=\"%s\" newsuperior=\"%s\"%s\n",
-					 (long long unsigned int)pb->pb_conn->c_connid,
+					 pb->pb_conn->c_connid,
 					 pb->pb_op->o_opid,
 					 dn,
 					 newrdn ? newrdn : "(null)",
@@ -527,7 +527,7 @@ op_shared_rename(Slapi_PBlock *pb, int passin_args)
 		if ( !internal_op ) {
 			slapi_log_error(SLAPI_LOG_ARGS, NULL, 
 				 "conn=%" NSPRIu64 " op=%d MODRDN invalid new RDN (\"%s\")\n",
-				 (long long unsigned int)pb->pb_conn->c_connid,
+				 pb->pb_conn->c_connid,
 				 pb->pb_op->o_opid,
 				 (NULL == newrdn) ? "(null)" : newrdn);
 		} else {
@@ -560,7 +560,7 @@ op_shared_rename(Slapi_PBlock *pb, int passin_args)
 		if (!internal_op) {
 			slapi_log_error(SLAPI_LOG_ARGS, NULL,
 				 "conn=%" NSPRIu64 " op=%d MODRDN invalid new superior (\"%s\")",
-				 (long long unsigned int)pb->pb_conn->c_connid,
+				 pb->pb_conn->c_connid,
 				 pb->pb_op->o_opid,
 				 newsuperior ? newsuperior : "(null)");
 		} else {
