@@ -907,10 +907,24 @@ slapi_attr_assertion2keys_sub_sv(
     char		*final,
     Slapi_Value	***ivals
 )
+{
+	return slapi_attr_assertion2keys_sub_sv_pb(NULL, sattr, initial, any, final, ivals);
+}
+
+int
+slapi_attr_assertion2keys_sub_sv_pb(
+    Slapi_PBlock *pb,
+    const Slapi_Attr *sattr,
+    char		*initial,
+    char		**any,
+    char		*final,
+    Slapi_Value	***ivals
+)
 {
 	int			rc;
 	Slapi_PBlock		pipb;
 	struct slapdplugin	*pi = NULL;
+	struct slapdplugin	*origpi = NULL;
 	IFP a2k_fn = NULL;
 
 	LDAPDebug( LDAP_DEBUG_FILTER,
@@ -927,13 +941,24 @@ slapi_attr_assertion2keys_sub_sv(
 		pi = sattr->a_plugin;
 		a2k_fn = sattr->a_plugin->plg_syntax_assertion2keys_sub;
 	}
-	pblock_init( &pipb );
-	slapi_pblock_set( &pipb, SLAPI_PLUGIN, pi );
+	if (NULL == pb) {
+		pblock_init( &pipb );
+		pb = &pipb;
+	} else {
+		/* back up the original slapdplugin if any */
+		slapi_pblock_get(pb, SLAPI_PLUGIN, &origpi);
+	}
+	slapi_pblock_set(pb, SLAPI_PLUGIN, pi);
 
 	rc = -1;	/* means no assertion2keys function */
 	*ivals = NULL;
 	if ( a2k_fn != NULL ) {
-		rc = (*a2k_fn)( &pipb, initial, any, final, ivals );
+		rc = (*a2k_fn)( pb, initial, any, final, ivals );
+	}
+
+	if (pb != &pipb) {
+		/* restore the original slapdplugin if pb is not local. */
+		slapi_pblock_set(pb, SLAPI_PLUGIN, origpi);
 	}
 
 	LDAPDebug( LDAP_DEBUG_FILTER,
