@@ -364,16 +364,18 @@ send_ldap_result_ext(
     BerElement          *ber
 )
 {
-	Connection	*conn = pb->pb_conn;
-	int		i, rc, logit = 0;
-	ber_tag_t	tag;
-	int             flush_ber_element = 1;
 	Slapi_Operation *operation;
-	const char *dn = NULL;
-	Slapi_DN *sdn = NULL;
-	int internal_op;
 	passwdPolicy *pwpolicy = NULL;
-	
+	Connection *conn = pb->pb_conn;
+	Slapi_DN *sdn = NULL;
+	const char *dn = NULL;
+	ber_tag_t	tag;
+	int flush_ber_element = 1;
+	ber_tag_t bind_method = 0;
+	int internal_op;
+	int i, rc, logit = 0;
+
+	slapi_pblock_get (pb, SLAPI_BIND_METHOD, &bind_method);
 	slapi_pblock_get (pb, SLAPI_OPERATION, &operation);
 
 	if (operation->o_status == SLAPI_OP_STATUS_RESULT_SENT) {
@@ -451,7 +453,7 @@ send_ldap_result_ext(
 
 	/* invalid password.  Update the password retry here */
 	/* put this here for now.  It could be a send_result pre-op plugin. */
-	if (err == LDAP_INVALID_CREDENTIALS) {
+	if ((err == LDAP_INVALID_CREDENTIALS) && (bind_method != LDAP_AUTH_SASL)) {
 		slapi_pblock_get( pb, SLAPI_TARGET_SDN, &sdn );
 		dn = slapi_sdn_get_dn(sdn);
 		pwpolicy = new_passwdPolicy(pb, dn);
@@ -1059,6 +1061,8 @@ encode_attr(
 
 #define LASTMODATTR( x )	(strcasecmp( x, "modifytimestamp" ) == 0 \
 				    || strcasecmp( x, "modifiersname" ) == 0 \
+				    || strcasecmp( x, "internalmodifytimestamp" ) == 0 \
+				    || strcasecmp( x, "internalmodifiersname" ) == 0 \
 				    || strcasecmp( x, "createtimestamp" ) == 0 \
 				    || strcasecmp( x, "creatorsname" ) == 0)
 
@@ -1957,6 +1961,9 @@ log_result( Slapi_PBlock *pb, Operation *op, int err, ber_tag_t tag, int nentrie
 	CSN *operationcsn = NULL;
 	char csn_str[CSN_STRSIZE + 5];
 	char etime[ETIME_BUFSIZ];
+	int pr_idx = -1;
+
+	slapi_pblock_get(pb, SLAPI_PAGED_RESULTS_INDEX, &pr_idx);
 
 	internal_op = operation_is_flag_set( op, OP_FLAG_INTERNAL );
 
@@ -1998,7 +2005,7 @@ log_result( Slapi_PBlock *pb, Operation *op, int err, ber_tag_t tag, int nentrie
 							  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
 							  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
 							  ", SASL bind in progress\n",
-							  (long long unsigned int)op->o_connid,
+							  op->o_connid,
 							  op->o_opid,
 							  err, tag, nentries,
 							  etime,
@@ -2030,7 +2037,7 @@ log_result( Slapi_PBlock *pb, Operation *op, int err, ber_tag_t tag, int nentrie
 							  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
 							  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
 							  " dn=\"%s\"\n",
-							  (long long unsigned int)op->o_connid,
+							  op->o_connid,
 							  op->o_opid,
 							  err, tag, nentries,
 							  etime,
@@ -2050,12 +2057,39 @@ log_result( Slapi_PBlock *pb, Operation *op, int err, ber_tag_t tag, int nentrie
 		}
 		slapi_ch_free((void**)&dn);
 	} else {
-		if ( !internal_op )
+		if (pr_idx > -1)
+		{
+			if ( !internal_op )
+			{
+				slapi_log_access( LDAP_DEBUG_STATS,
+								  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
+								  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
+								  " pr_idx=%d\n",
+								  op->o_connid, 
+								  op->o_opid,
+								  err, tag, nentries, 
+								  etime, 
+								  notes_str, csn_str, pr_idx );
+			}
+			else
+			{
+				slapi_log_access( LDAP_DEBUG_ARGS,
+								  "conn=%s op=%d RESULT err=%d"
+								  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s"
+								  " pr_idx=%d\n",
+								  LOG_INTERNAL_OP_CON_ID,
+								  LOG_INTERNAL_OP_OP_ID,
+								  err, tag, nentries, 
+								  etime, 
+								  notes_str, csn_str, pr_idx );
+			}
+		}
+		else if ( !internal_op )
 		{
 			slapi_log_access( LDAP_DEBUG_STATS,
 							  "conn=%" NSPRIu64 " op=%d RESULT err=%d"
 							  " tag=%" BERTAG_T " nentries=%d etime=%s%s%s\n",
-							  (long long unsigned int)op->o_connid,
+							  op->o_connid,
 							  op->o_opid,
 							  err, tag, nentries,
 							  etime,
@@ -2079,7 +2113,8 @@ log_result( Slapi_PBlock *pb, Operation *op, int err, ber_tag_t tag, int nentrie
 			slapi_pblock_get( pb, SLAPI_OPERATION_TYPE, &optype );
 			if(optype == SLAPI_OPERATION_SEARCH && /* search, */
 			   strcmp(notes_str,"") &&  /* that's unindexed, */
-			   !(config_get_accesslog_level() & LDAP_DEBUG_ARGS)) /* and not logged in access log */
+			   !(config_get_accesslog_level() & LDAP_DEBUG_ARGS) && /* and not logged in access log */
+			   !(op->o_flags & SLAPI_OP_FLAG_IGNORE_UNINDEXED) ) /* and not ignoring unindexed search */
 			{
 				struct slapdplugin *plugin = NULL;
 				struct slapi_componentid *cid = NULL;
@@ -2118,7 +2153,7 @@ log_entry( Operation *op, Slapi_Entry *e )
 	if ( !internal_op )
 	{
 		slapi_log_access( LDAP_DEBUG_STATS2, "conn=%" NSPRIu64 " op=%d ENTRY dn=\"%s\"\n",
-			(long long unsigned int)op->o_connid, op->o_opid,
+			op->o_connid, op->o_opid,
 			slapi_entry_get_dn_const(e));
 	}
 	else
@@ -2143,7 +2178,7 @@ log_referral( Operation *op )
 	if ( !internal_op )
 	{
 		slapi_log_access( LDAP_DEBUG_STATS2, "conn=%" NSPRIu64 " op=%d REFERRAL\n",
-			(long long unsigned int)op->o_connid, op->o_opid );
+			op->o_connid, op->o_opid );
 	}
 	else
 	{
