@@ -64,7 +64,7 @@
 #include <string.h>
 #include <errno.h>
 
-#define NEED_TOK_DES /* defines tokDes and ptokDes - see slap.h */
+#define NEED_TOK_PBE /* defines tokPBE and ptokPBE - see slap.h */
 #include "slap.h"
 
 #include "svrcore.h"
@@ -81,17 +81,34 @@
 #endif
 
 #if NSS_VMAJOR * 100 + NSS_VMINOR >= 315
+/* TLS1.2 is defined in RFC5246. */
 #define NSS_TLS12 1
 #elif NSS_VMAJOR * 100 + NSS_VMINOR >= 314
+/* TLS1.1 is defined in RFC4346. */
 #define NSS_TLS11 1
 #else
 #define NSS_TLS10 1
 #endif
 
+/******************************************************************************
+ * Default SSL Version Rule
+ * Old SSL version attributes:
+ *   nsSSL3: off -- nsSSL3 == SSL_LIBRARY_VERSION_3_0
+ *   nsTLS1: on  -- nsTLS1 == SSL_LIBRARY_VERSION_TLS_1_0 and greater
+ *   Note: TLS1.0 is defined in RFC2246, which is close to SSL 3.0.
+ * New SSL version attributes:
+ *   sslVersionMin: TLS1.0
+ *   sslVersionMax: max ssl version supported by NSS
+ ******************************************************************************/
+
+#define DEFVERSION "TLS1.0"
+#define CURRENT_DEFAULT_SSL_VERSION SSL_LIBRARY_VERSION_TLS_1_0
+
 extern char* slapd_SSL3ciphers;
 extern symbol_t supported_ciphers[];
 #if !defined(NSS_TLS10) /* NSS_TLS11 or newer */
-static SSLVersionRange    enabledNSSVersions;
+static SSLVersionRange enabledNSSVersions;
+static SSLVersionRange slapdNSSVersions;
 #endif
 
 /* dongle_file_name is set in slapd_nss_init when we set the path for the
@@ -238,6 +255,23 @@ static lookup_cipher _lookup_cipher[] = {
     {NULL, NULL}
 };
 
+/* E.g., "SSL3", "TLS1.2", "Unknown SSL version: 0x0" */
+#define VERSION_STR_LENGTH 64
+
+/* Supported SSL versions  */
+/* nsSSL2: on -- we don't allow this any more. */
+PRBool enableSSL2 = PR_FALSE;
+/*
+ * nsSSL3: on -- disable SSLv3 by default.
+ * Corresonding to SSL_LIBRARY_VERSION_3_0
+ */
+PRBool enableSSL3 = PR_FALSE;
+/*
+ * nsTLS1: on -- enable TLS1 by default.
+ * Corresonding to SSL_LIBRARY_VERSION_TLS_1_0 and greater.
+ */
+PRBool enableTLS1 = PR_TRUE;
+
 static void
 slapd_SSL_report(int degree, char *fmt, va_list args)
 {
@@ -372,6 +406,36 @@ cipher_check_fips(int idx, char ***suplist, char ***unsuplist)
     return rc;
 }
 
+int
+getSSLVersionInfo(int *ssl2, int *ssl3, int *tls1)
+{
+    if (!slapd_ssl_listener_is_initialized()) {
+        return -1;
+    }
+    *ssl2 = enableSSL2;
+    *ssl3 = enableSSL3;
+    *tls1 = enableTLS1;
+    return 0;
+}
+
+int
+getSSLVersionRange(char **min, char **max)
+{
+    if (!slapd_ssl_listener_is_initialized()) {
+        return -1;
+    }
+    if ((NULL == min) || (NULL == max)) {
+        return -1;
+    }
+#if defined(NSS_TLS10)
+    return -1; /* not supported */
+#else /* NSS_TLS11 or newer */
+    *min = slapi_getSSLVersion_str(slapdNSSVersions.min, NULL, 0);
+    *max = slapi_getSSLVersion_str(slapdNSSVersions.max, NULL, 0);
+    return 0;
+#endif
+}
+
 static void
 _conf_init_ciphers()
 {
@@ -802,121 +866,168 @@ warn_if_no_key_file(const char *dir, int no_log)
 }
 
 #if !defined(NSS_TLS10) /* NSS_TLS11 or newer */
-typedef struct _nss_version_list {
-    PRUint16 vnum;
-    char* vname;
-} NSSVersion_list;
-NSSVersion_list _NSSVersion_list[] =
+/* 
+ * If non NULL buf and positive bufsize is given,
+ * the memory is used to store the version string.
+ * Otherwise, the memory for the string is allocated.
+ * The latter case, caller is responsible to free it.
+ */
+char *
+slapi_getSSLVersion_str(PRUint16 vnum, char *buf, size_t bufsize)
 {
-    {SSL_LIBRARY_VERSION_2,       "SSL2"},
-    {SSL_LIBRARY_VERSION_3_0,     "SSL3"},
-    {SSL_LIBRARY_VERSION_TLS_1_0, "TLS1.0"},
-    {SSL_LIBRARY_VERSION_TLS_1_1, "TLS1.1"},
-#if defined(NSS_TLS12)
-    {SSL_LIBRARY_VERSION_TLS_1_2, "TLS1.2"},
-#endif
-    {0, "unknown"}
-};
+    char *vstr = buf;
+    if (vnum >= SSL_LIBRARY_VERSION_3_0) {
+        if (vnum == SSL_LIBRARY_VERSION_3_0) { /* SSL3 */
+            if (buf && bufsize) { 
+                PR_snprintf(buf, bufsize, "SSL3"); 
+            } else { 
+                vstr = slapi_ch_smprintf("SSL3"); 
+            } 
+        } else { /* TLS v X.Y */
+            const char *TLSFMT = "TLS%d.%d";
+            int minor_offset = 0; /* e.g. 0x0401 -> TLS v 2.1, not 2.0 */
 
-static char *
-getNSSVersion_str(PRUint16 vnum)
-{
-    NSSVersion_list *nvlp = NULL;
-    char *vstr = "none";
-    if (vnum) {
-        for (nvlp = _NSSVersion_list; nvlp && nvlp->vnum; nvlp++) {
-            if (nvlp->vnum == vnum) {
-                vstr = nvlp->vname;
-                break;
+            if ((vnum & SSL_LIBRARY_VERSION_3_0) == SSL_LIBRARY_VERSION_3_0) {
+                minor_offset = 1; /* e.g. 0x0301 -> TLS v 1.0, not 1.1 */
             }
+            if (buf && bufsize) { 
+                PR_snprintf(buf, bufsize, TLSFMT, (vnum >> 8) - 2, (vnum & 0xff) - minor_offset); 
+            } else { 
+                vstr = slapi_ch_smprintf(TLSFMT, (vnum >> 8) - 2, (vnum & 0xff) - minor_offset); 
+            }
+        }
+    } else if (vnum == SSL_LIBRARY_VERSION_2) { /* SSL2 */
+        if (buf && bufsize) {
+            PR_snprintf(buf, bufsize, "SSL2");
+        } else {
+            vstr = slapi_ch_smprintf("SSL2");
+        }
+    } else {
+        if (buf && bufsize) {
+            PR_snprintf(buf, bufsize, "Unknown SSL version: 0x%x", vnum); 
+        } else {
+            vstr = slapi_ch_smprintf("Unknown SSL version: 0x%x", vnum);
         }
     }
     return vstr;
 }
 
-/* restrict SSLVersionRange with the existing SSL config params (nsSSL3, nsTLS1) */
+#define SSLVGreater(x, y) (((x) > (y)) ? (x) : (y))
+
+/*
+ * Check the SSLVersionRange and the old style config params (nsSSL3, nsTLS1) .
+ * If there are conflicts, choose the secure setting.
+ */
 static void
-restrict_SSLVersionRange(SSLVersionRange *sslversion, PRBool enableSSL3, PRBool enableTLS1)
+restrict_SSLVersionRange(void)
 {
-    int rc = 0;
+    char mymin[VERSION_STR_LENGTH], mymax[VERSION_STR_LENGTH];
+    char emin[VERSION_STR_LENGTH], emax[VERSION_STR_LENGTH];
+    (void) slapi_getSSLVersion_str(slapdNSSVersions.min, mymin, sizeof(mymin));
+    (void) slapi_getSSLVersion_str(slapdNSSVersions.max, mymax, sizeof(mymax));
+    (void) slapi_getSSLVersion_str(enabledNSSVersions.max, emax, sizeof(emax));
+    (void) slapi_getSSLVersion_str(enabledNSSVersions.min, emin, sizeof(emin));
+    if (slapdNSSVersions.min > slapdNSSVersions.max) {
+        slapd_SSL_warn("Invalid configured SSL range: min: %s, max: %s; "
+                       "Resetting the max to the supported max SSL version: %s.",
+                       mymin, mymax, emax);
+        slapdNSSVersions.max = enabledNSSVersions.max;
+    }
     if (enableSSL3) {
+        slapd_SSL_warn("Found unsecure configuration: nsSSL3: on; "
+                       "We strongly recommend to disable nsSSL3 in %s.", configDN);
         if (enableTLS1) {
-            /* no restriction */
-            ;
+            if (slapdNSSVersions.min >= CURRENT_DEFAULT_SSL_VERSION) {
+                slapd_SSL_warn("Configured range: min: %s, max: %s; "
+                               "but both nsSSL3 and nsTLS1 are on. "
+                               "Respect the supported range.",
+                               mymin, mymax);
+                enableSSL3 = PR_FALSE;
+            }
+            if (slapdNSSVersions.max < CURRENT_DEFAULT_SSL_VERSION) {
+                slapd_SSL_warn("Configured range: min: %s, max: %s; "
+                               "but both nsSSL3 and nsTLS1 are on. "
+                               "Resetting the max to the supported max SSL version: %s.",
+                               mymin, mymax, emax);
+                slapdNSSVersions.max = enabledNSSVersions.max;
+            }
         } else {
-            if (enabledNSSVersions.min > SSL_LIBRARY_VERSION_3_0) {
-                slapd_SSL_warn("Security Initialization: "
-                               "Supported range: min: %s, max: %s; "
-                               "but the SSL configuration of the server disables nsTLS1. "
-                               "Ignoring nsTLS1: off\n",
-                               getNSSVersion_str(enabledNSSVersions.min),
-                               getNSSVersion_str(enabledNSSVersions.max));
-                rc = 1;
-            } else if (sslversion->min > SSL_LIBRARY_VERSION_3_0) {
-                slapd_SSL_warn("Security Initialization: "
-                               "Configured range: min: %s, max: %s; "
-                               "but the SSL configuration of the server disables nsTLS1. "
-                               "Ignoring nsTLS1: off\n",
-                               getNSSVersion_str(sslversion->min),
-                               getNSSVersion_str(sslversion->max));
-                rc = 1;
-            } else if (sslversion->max < SSL_LIBRARY_VERSION_3_0) {
-                slapd_SSL_warn("Security Initialization: "
-                               "Configured range: min: %s, max: %s; "
-                               "but the SSL configuration of the server enabled nsSSL3. "
-                               "Ignoring max: %s\n",
-                               getNSSVersion_str(sslversion->min),
-                               getNSSVersion_str(sslversion->max),
-                               getNSSVersion_str(sslversion->max));
-                sslversion->min = SSL_LIBRARY_VERSION_3_0; /* don't enable SSL2 */
-                sslversion->max = SSL_LIBRARY_VERSION_3_0;
-                rc = 1;
+            /* nsTLS1 is explicitly set to off. */
+            if (enabledNSSVersions.min >= CURRENT_DEFAULT_SSL_VERSION) {
+                slapd_SSL_warn("Supported range: min: %s, max: %s; "
+                               "but nsSSL3 is on and nsTLS1 is off. "
+                               "Respect the supported range.",
+                               emin, emax);
+                slapdNSSVersions.min = SSLVGreater(slapdNSSVersions.min, enabledNSSVersions.min);
+                enableSSL3 = PR_FALSE;
+                enableTLS1 = PR_TRUE;
+            } else if (slapdNSSVersions.min >= CURRENT_DEFAULT_SSL_VERSION) { 
+                slapd_SSL_warn("Configured range: min: %s, max: %s; "
+                               "but nsSSL3 is on and nsTLS1 is off. "
+                               "Respect the configured range.",
+                               mymin, mymax);
+                enableSSL3 = PR_FALSE;
+                enableTLS1 = PR_TRUE;
+            } else if (slapdNSSVersions.min < CURRENT_DEFAULT_SSL_VERSION) {
+                slapd_SSL_warn("Too low configured range: min: %s, max: %s; "
+                               "We strongly recommend to set sslVersionMin higher than %s.",
+                               mymin, mymax, DEFVERSION);
             } else {
-                sslversion->min = SSL_LIBRARY_VERSION_3_0; /* don't enable SSL2 */
-                sslversion->max = SSL_LIBRARY_VERSION_3_0;
+                /* 
+                 * slapdNSSVersions.min < SSL_LIBRARY_VERSION_TLS_1_0 &&
+                 * slapdNSSVersions.max >= SSL_LIBRARY_VERSION_TLS_1_1
+                 */
+                slapd_SSL_warn("Configured range: min: %s, max: %s; "
+                               "but nsSSL3 is on and nsTLS1 is off. "
+                               "Respect the configured range.",
+                               mymin, mymax);
+                enableTLS1 = PR_TRUE;
             }
         }
     } else {
         if (enableTLS1) {
-            if (enabledNSSVersions.max < SSL_LIBRARY_VERSION_TLS_1_0) {
-                slapd_SSL_warn("Security Initialization: "
-                               "Supported range: min: %s, max: %s; "
-                               "but the SSL configuration of the server disables nsSSL3. ",
-                               "Ignoring nsSSL3: off\n",
-                               getNSSVersion_str(enabledNSSVersions.min),
-                               getNSSVersion_str(enabledNSSVersions.max));
-                sslversion->min = SSL_LIBRARY_VERSION_3_0; /* don't enable SSL2 */
-                sslversion->max = SSL_LIBRARY_VERSION_3_0;
-                rc = 1;
-            } else if (sslversion->max < SSL_LIBRARY_VERSION_TLS_1_0) {
-                slapd_SSL_warn("Security Initialization: "
-                               "Configured range: min: %s, max: %s; "
-                               "but the SSL configuration of the server disables nsSSL3. "
-                               "Ignoring nsSSL3: off\n",
-                               getNSSVersion_str(sslversion->min),
-                               getNSSVersion_str(sslversion->max));
-                sslversion->min = SSL_LIBRARY_VERSION_3_0; /* don't enable SSL2 */
-                sslversion->max = SSL_LIBRARY_VERSION_3_0;
-                rc = 1;
-            } else if (sslversion->min < SSL_LIBRARY_VERSION_TLS_1_0) {
-                sslversion->min = SSL_LIBRARY_VERSION_TLS_1_0;
+            if (enabledNSSVersions.max < CURRENT_DEFAULT_SSL_VERSION) {
+                /* TLS1 is on, but TLS1 is not supported by NSS.  */
+                slapd_SSL_warn("Supported range: min: %s, max: %s; "
+                               "Setting the version range based upon the supported range.",
+                               emin, emax);
+                slapdNSSVersions.max = enabledNSSVersions.max;
+                slapdNSSVersions.min = enabledNSSVersions.min;
+                enableSSL3 = PR_TRUE;
+                enableTLS1 = PR_FALSE;
+            } else if ((slapdNSSVersions.max < CURRENT_DEFAULT_SSL_VERSION) ||
+                       (slapdNSSVersions.min < CURRENT_DEFAULT_SSL_VERSION)) {
+                slapdNSSVersions.max = enabledNSSVersions.max;
+                slapdNSSVersions.min = SSLVGreater(CURRENT_DEFAULT_SSL_VERSION, enabledNSSVersions.min);
+                slapd_SSL_warn("nsTLS1 is on, but the version range is lower than \"%s\"; "
+                               "Configuring the version range as default min: %s, max: %s.",
+                               DEFVERSION, DEFVERSION, emax);
+            } else {
+                /* 
+                 * slapdNSSVersions.min >= SSL_LIBRARY_VERSION_TLS_1_0 &&
+                 * slapdNSSVersions.max >= SSL_LIBRARY_VERSION_TLS_1_0
+                 */
+                ;
             }
         } else {
-            slapd_SSL_warn("Security Initialization: "
-                            "Supported range: min: %s, max: %s; "
-                            "but the SSL configuration of the server disables nsSSL3 and nsTLS1. "
-                            "Ignoring nsSSL3: off and nsTLS1: off\n",
-                            getNSSVersion_str(enabledNSSVersions.min),
-                            getNSSVersion_str(enabledNSSVersions.max));
-            rc = 1;
+            slapd_SSL_warn("Supported range: min: %s, max: %s; "
+                           "Respect the configured range.",
+                           emin, emax);
+            /* nsTLS1 is explicitly set to off. */
+            if (slapdNSSVersions.min >= CURRENT_DEFAULT_SSL_VERSION) {
+                enableTLS1 = PR_TRUE;
+            } else if (slapdNSSVersions.max < CURRENT_DEFAULT_SSL_VERSION) {
+                enableSSL3 = PR_TRUE;
+            } else {
+                /* 
+                 * slapdNSSVersions.min < SSL_LIBRARY_VERSION_TLS_1_0 &&
+                 * slapdNSSVersions.max >= SSL_LIBRARY_VERSION_TLS_1_0
+                 */
+                enableSSL3 = PR_TRUE;
+                enableTLS1 = PR_TRUE;
+            }
         }
     }
-    if (0 == rc) {
-        slapi_log_error(SLAPI_LOG_FATAL, "SSL Initialization",
-                        "SSL version range: min: %s, max: %s\n",
-                        getNSSVersion_str(sslversion->min),
-                        getNSSVersion_str(sslversion->max));
-    }
 }
 #endif
 
@@ -945,13 +1056,15 @@ slapd_nss_init(int init_ssl, int config_available)
 	char *keydb_file_name = NULL;
 	char *secmoddb_file_name = NULL;
 #if !defined(NSS_TLS10) /* NSS_TLS11 or newer */
+	char emin[VERSION_STR_LENGTH], emax[VERSION_STR_LENGTH];
 	/* Get the range of the supported SSL version */
 	SSL_VersionRangeGetSupported(ssl_variant_stream, &enabledNSSVersions);
 	
+	(void) slapi_getSSLVersion_str(enabledNSSVersions.min, emin, sizeof(emin));
+	(void) slapi_getSSLVersion_str(enabledNSSVersions.max, emax, sizeof(emax));
 	slapi_log_error(SLAPI_LOG_CONFIG, "SSL Initialization",
-	                "supported range: min: %s, max: %s\n",
-	                getNSSVersion_str(enabledNSSVersions.min),
-	                getNSSVersion_str(enabledNSSVersions.max));
+	                "supported range by NSS: min: %s, max: %s\n",
+	                emin, emax);
 #endif
 
 	/* set in slapd_bootstrap_config,
@@ -995,7 +1108,7 @@ slapd_nss_init(int init_ssl, int config_available)
 	/******** Initialise NSS *********/
     
 	nssFlags &= (~NSS_INIT_READONLY);
-	slapd_pk11_configurePKCS11(NULL, NULL, tokDes, ptokDes, NULL, NULL, NULL, NULL, 0, 0 );
+	slapd_pk11_configurePKCS11(NULL, NULL, tokPBE, ptokPBE, NULL, NULL, NULL, NULL, 0, 0 );
 	secStatus = NSS_Initialize(certdir, NULL, NULL, "secmod.db", nssFlags);
 
 	dongle_file_name = PR_smprintf("%s/pin.txt", certdir);
@@ -1120,11 +1233,8 @@ slapd_ssl_init()
     PRErrorCode errorCode;
     char ** family_list;
     char *val = NULL;
-    char cipher_string[1024];
-    int rv = 0;
     PK11SlotInfo *slot;
     Slapi_Entry *entry = NULL;
-    int allowweakcipher = CIPHER_SET_DEFAULTWEAKCIPHER;
 
     /* Get general information */
 
@@ -1162,23 +1272,6 @@ slapd_ssl_init()
         freeConfigEntry( &entry );
         return -1;
     }
-
-    val = slapi_entry_attr_get_charptr(entry, "allowWeakCipher");
-    if (val) {
-        if (!PL_strcasecmp(val, "off") || !PL_strcasecmp(val, "false") || 
-                !PL_strcmp(val, "0") || !PL_strcasecmp(val, "no")) {
-            allowweakcipher = CIPHER_SET_DISALLOWWEAKCIPHER;
-        } else if (!PL_strcasecmp(val, "on") || !PL_strcasecmp(val, "true") || 
-                !PL_strcmp(val, "1") || !PL_strcasecmp(val, "yes")) {
-            allowweakcipher = CIPHER_SET_ALLOWWEAKCIPHER;
-        } else {
-            slapd_SSL_warn("The value of allowWeakCipher \"%s\" in "
-                           "cn=encryption,cn=config is invalid. "
-                           "Ignoring it and set it to default.", val);
-        }
-    }
-    slapi_ch_free((void **) &val);
- 
     if ((family_list = getChildren(configDN))) {
         char **family;
         char *token;
@@ -1249,35 +1342,12 @@ slapd_ssl_init()
         freeConfigEntry( &entry );
     }
 
-    /* ugaston- Cipher preferences must be set before any sslSocket is created
-     * for such sockets to take preferences into account.
-     */
-
-    /* Step Three.5: Set SSL cipher preferences */
-    *cipher_string = 0;
-    if(ciphers && (*ciphers) && PL_strcmp(ciphers, "blank"))
-         PL_strncpyz(cipher_string, ciphers, sizeof(cipher_string));
-    slapi_ch_free((void **) &ciphers);
-
-    if ( NULL != (val = _conf_setciphers(cipher_string, allowweakcipher)) ) {
-        errorCode = PR_GetError();
-        slapd_SSL_warn("Security Initialization: Failed to set SSL cipher "
-            "preference information: %s (" SLAPI_COMPONENT_NAME_NSPR " error %d - %s)", 
-            val, errorCode, slapd_pr_strerror(errorCode));
-        rv = 3;
-        slapi_ch_free((void **) &val);
-    }
-
     freeConfigEntry( &entry );
  
     /* Introduce a way of knowing whether slapd_ssl_init has
      * already been executed. */
     _security_library_initialized = 1; 
 
-    if ( rv != 0 ) {
-        return rv;
-    }
-
     return 0;
 }
 
@@ -1296,34 +1366,37 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
 {
     char *vp, *endp;
     int vnum;
+    char emin[VERSION_STR_LENGTH], emax[VERSION_STR_LENGTH];
 
     if (NULL == rval) {
         return 1;
     }
+    (void) slapi_getSSLVersion_str(enabledNSSVersions.min, emin, sizeof(emin));
+    (void) slapi_getSSLVersion_str(enabledNSSVersions.max, emax, sizeof(emax));
     if (!strncasecmp(val, SSLSTR, SSLLEN)) { /* ssl# */
         vp = val + SSLLEN;
         vnum = strtol(vp, &endp, 10);
         if (2 == vnum) {
             if (ismin) {
                 if (enabledNSSVersions.min > SSL_LIBRARY_VERSION_2) {
-                   slapd_SSL_warn("Security Initialization: The value of sslVersionMin "
-                                  "\"%s\" is lower than the supported version; "
-                                  "the default value \"%s\" is used.\n",
-                                  val, getNSSVersion_str(enabledNSSVersions.min));
-                   (*rval) = enabledNSSVersions.min;
+                    slapd_SSL_warn("Security Initialization: The value of sslVersionMin "
+                                   "\"%s\" is lower than the supported version; "
+                                   "the default value \"%s\" is used.",
+                                   val, emin);
+                    (*rval) = enabledNSSVersions.min;
                 } else {
-                   (*rval) = SSL_LIBRARY_VERSION_2;
+                    (*rval) = SSL_LIBRARY_VERSION_2;
                 }
             } else {
                 if (enabledNSSVersions.max < SSL_LIBRARY_VERSION_2) {
                     /* never happens */
                     slapd_SSL_warn("Security Initialization: The value of sslVersionMax "
-                                   "\"%s\" is higher than the supported version; "
-                                   "the default value \"%s\" is used.\n",
-                                   val, getNSSVersion_str(enabledNSSVersions.max));
-                   (*rval) = enabledNSSVersions.max;
+                                    "\"%s\" is higher than the supported version; "
+                                    "the default value \"%s\" is used.",
+                                    val, emax);
+                    (*rval) = enabledNSSVersions.max;
                 } else {
-                   (*rval) = SSL_LIBRARY_VERSION_2;
+                    (*rval) = SSL_LIBRARY_VERSION_2;
                 }
             }
         } else if (3 == vnum) {
@@ -1331,8 +1404,8 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
                 if (enabledNSSVersions.min > SSL_LIBRARY_VERSION_3_0) {
                     slapd_SSL_warn("Security Initialization: The value of sslVersionMin "
                                    "\"%s\" is lower than the supported version; "
-                                   "the default value \"%s\" is used.\n",
-                                   val, getNSSVersion_str(enabledNSSVersions.min));
+                                   "the default value \"%s\" is used.",
+                                   val, emin);
                    (*rval) = enabledNSSVersions.min;
                 } else {
                    (*rval) = SSL_LIBRARY_VERSION_3_0;
@@ -1342,8 +1415,8 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
                     /* never happens */
                     slapd_SSL_warn("Security Initialization: The value of sslVersionMax "
                                    "\"%s\" is higher than the supported version; "
-                                   "the default value \"%s\" is used.\n",
-                                   val, getNSSVersion_str(enabledNSSVersions.max));
+                                   "the default value \"%s\" is used.",
+                                   val, emax);
                     (*rval) = enabledNSSVersions.max;
                 } else {
                     (*rval) = SSL_LIBRARY_VERSION_3_0;
@@ -1352,13 +1425,13 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
         } else {
             if (ismin) {
                 slapd_SSL_warn("Security Initialization: The value of sslVersionMin "
-                               "\"%s\" is invalid; the default value \"%s\" is used.\n",
-                               val, getNSSVersion_str(enabledNSSVersions.min));
+                               "\"%s\" is invalid; the default value \"%s\" is used.",
+                               val, emin);
                 (*rval) = enabledNSSVersions.min;
             } else {
                 slapd_SSL_warn("Security Initialization: The value of sslVersionMax "
-                               "\"%s\" is invalid; the default value \"%s\" is used.\n",
-                               val, getNSSVersion_str(enabledNSSVersions.max));
+                               "\"%s\" is invalid; the default value \"%s\" is used.",
+                               val, emax);
                 (*rval) = enabledNSSVersions.max;
             }
         }
@@ -1368,25 +1441,25 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
         sscanf(vp, "%4f", &tlsv);
         if (tlsv < 1.1) { /* TLS1.0 */
             if (ismin) {
-                if (enabledNSSVersions.min > SSL_LIBRARY_VERSION_TLS_1_0) {
+                if (enabledNSSVersions.min > CURRENT_DEFAULT_SSL_VERSION) {
                     slapd_SSL_warn("Security Initialization: The value of sslVersionMin "
                                    "\"%s\" is lower than the supported version; "
-                                   "the default value \"%s\" is used.\n",
-                                   val, getNSSVersion_str(enabledNSSVersions.min));
+                                   "the default value \"%s\" is used.",
+                                   val, emin);
                    (*rval) = enabledNSSVersions.min;
                 } else {
-                   (*rval) = SSL_LIBRARY_VERSION_TLS_1_0;
+                   (*rval) = CURRENT_DEFAULT_SSL_VERSION;
                 }
             } else {
-                if (enabledNSSVersions.max < SSL_LIBRARY_VERSION_TLS_1_0) {
+                if (enabledNSSVersions.max < CURRENT_DEFAULT_SSL_VERSION) {
                     /* never happens */
                     slapd_SSL_warn("Security Initialization: The value of sslVersionMax "
                                    "\"%s\" is higher than the supported version; "
-                                   "the default value \"%s\" is used.\n",
-                                   val, getNSSVersion_str(enabledNSSVersions.max));
+                                   "the default value \"%s\" is used.",
+                                   val, emax);
                     (*rval) = enabledNSSVersions.max;
                 } else {
-                    (*rval) = SSL_LIBRARY_VERSION_TLS_1_0;
+                    (*rval) = CURRENT_DEFAULT_SSL_VERSION;
                 }
             }
         } else if (tlsv < 1.2) { /* TLS1.1 */
@@ -1394,8 +1467,8 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
                 if (enabledNSSVersions.min > SSL_LIBRARY_VERSION_TLS_1_1) {
                     slapd_SSL_warn("Security Initialization: The value of sslVersionMin "
                                    "\"%s\" is lower than the supported version; "
-                                   "the default value \"%s\" is used.\n",
-                                   val, getNSSVersion_str(enabledNSSVersions.min));
+                                   "the default value \"%s\" is used.",
+                                   val, emin);
                    (*rval) = enabledNSSVersions.min;
                 } else {
                    (*rval) = SSL_LIBRARY_VERSION_TLS_1_1;
@@ -1405,8 +1478,8 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
                     /* never happens */
                     slapd_SSL_warn("Security Initialization: The value of sslVersionMax "
                                    "\"%s\" is higher than the supported version; "
-                                   "the default value \"%s\" is used.\n",
-                                   val, getNSSVersion_str(enabledNSSVersions.max));
+                                   "the default value \"%s\" is used.",
+                                   val, emax);
                     (*rval) = enabledNSSVersions.max;
                 } else {
                     (*rval) = SSL_LIBRARY_VERSION_TLS_1_1;
@@ -1418,8 +1491,8 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
                 if (enabledNSSVersions.min > SSL_LIBRARY_VERSION_TLS_1_2) {
                     slapd_SSL_warn("Security Initialization: The value of sslVersionMin "
                                    "\"%s\" is lower than the supported version; "
-                                   "the default value \"%s\" is used.\n",
-                                   val, getNSSVersion_str(enabledNSSVersions.min));
+                                   "the default value \"%s\" is used.",
+                                   val, emin);
                    (*rval) = enabledNSSVersions.min;
                 } else {
                    (*rval) = SSL_LIBRARY_VERSION_TLS_1_2;
@@ -1429,8 +1502,8 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
                     /* never happens */
                     slapd_SSL_warn("Security Initialization: The value of sslVersionMax "
                                    "\"%s\" is higher than the supported version; "
-                                   "the default value \"%s\" is used.\n",
-                                   val, getNSSVersion_str(enabledNSSVersions.max));
+                                   "the default value \"%s\" is used.",
+                                   val, emax);
                     (*rval) = enabledNSSVersions.max;
                 } else {
                     (*rval) = SSL_LIBRARY_VERSION_TLS_1_2;
@@ -1441,27 +1514,27 @@ set_NSS_version(char *val, PRUint16 *rval, int ismin)
             if (ismin) {
                 slapd_SSL_warn("Security Initialization: The value of sslVersionMin "
                                "\"%s\" is out of the range of the supported version; "
-                               "the default value \"%s\" is used.\n",
-                               val, getNSSVersion_str(enabledNSSVersions.min));
+                               "the default value \"%s\" is used.",
+                               val, emin);
                 (*rval) = enabledNSSVersions.min;
             } else {
                 slapd_SSL_warn("Security Initialization: The value of sslVersionMax "
                                "\"%s\" is out of the range of the supported version; "
-                               "the default value \"%s\" is used.\n",
-                               val, getNSSVersion_str(enabledNSSVersions.min));
+                               "the default value \"%s\" is used.",
+                               val, emax);
                 (*rval) = enabledNSSVersions.max;
             }
         }
     } else {
         if (ismin) {
             slapd_SSL_warn("Security Initialization: The value of sslVersionMin "
-                           "\"%s\" is invalid; the default value \"%s\" is used.\n",
-                           val, getNSSVersion_str(enabledNSSVersions.min));
+                           "\"%s\" is invalid; the default value \"%s\" is used.",
+                           val, emin);
             (*rval) = enabledNSSVersions.min;
         } else {
             slapd_SSL_warn("Security Initialization: The value of sslVersionMax "
-                           "\"%s\" is invalid; the default value \"%s\" is used.\n",
-                           val, getNSSVersion_str(enabledNSSVersions.min));
+                           "\"%s\" is invalid; the default value \"%s\" is used.",
+                           val, emax);
             (*rval) = enabledNSSVersions.max;
         }
     }
@@ -1490,14 +1563,15 @@ slapd_ssl_init2(PRFileDesc **fd, int startTLS)
     int slapd_SSLclientAuth;
     char* tmpDir;
     Slapi_Entry *e = NULL;
-    PRBool enableSSL2 = PR_FALSE;
-    PRBool enableSSL3 = PR_TRUE;
-    PRBool enableTLS1 = PR_TRUE;
     PRBool fipsMode = PR_FALSE;
 #if !defined(NSS_TLS10) /* NSS_TLS11 or newer */
     PRUint16 NSSVersionMin = enabledNSSVersions.min;
     PRUint16 NSSVersionMax = enabledNSSVersions.max;
+    char mymin[VERSION_STR_LENGTH], mymax[VERSION_STR_LENGTH];
+    char newmax[VERSION_STR_LENGTH];
 #endif
+    char cipher_string[1024];
+    int allowweakcipher = CIPHER_SET_DEFAULTWEAKCIPHER;
 
     /* turn off the PKCS11 pin interactive mode */
 #ifndef _WIN32
@@ -1513,6 +1587,45 @@ slapd_ssl_init2(PRFileDesc **fd, int startTLS)
 
     errorbuf[0] = '\0';
 
+    /*
+     * Cipher preferences must be set before any sslSocket is created
+     * for such sockets to take preferences into account.
+     */
+    getConfigEntry(configDN, &e);
+    if (e == NULL) {
+        slapd_SSL_warn("Security Initialization: Failed get config entry %s", configDN);
+        return 1;
+    }
+    val = slapi_entry_attr_get_charptr(e, "allowWeakCipher");
+    if (val) {
+        if (!PL_strcasecmp(val, "off") || !PL_strcasecmp(val, "false") || 
+                !PL_strcmp(val, "0") || !PL_strcasecmp(val, "no")) {
+            allowweakcipher = CIPHER_SET_DISALLOWWEAKCIPHER;
+        } else if (!PL_strcasecmp(val, "on") || !PL_strcasecmp(val, "true") || 
+                !PL_strcmp(val, "1") || !PL_strcasecmp(val, "yes")) {
+            allowweakcipher = CIPHER_SET_ALLOWWEAKCIPHER;
+        } else {
+            slapd_SSL_warn("The value of allowWeakCipher \"%s\" in %s is invalid.",
+                           "Ignoring it and set it to default.", val, configDN);
+        }
+    }
+    slapi_ch_free((void **) &val);
+
+    /* Set SSL cipher preferences */
+    *cipher_string = 0;
+    if(ciphers && (*ciphers) && PL_strcmp(ciphers, "blank"))
+         PL_strncpyz(cipher_string, ciphers, sizeof(cipher_string));
+    slapi_ch_free((void **) &ciphers);
+
+    if ( NULL != (val = _conf_setciphers(cipher_string, allowweakcipher)) ) {
+        errorCode = PR_GetError();
+        slapd_SSL_warn("Security Initialization: Failed to set SSL cipher "
+            "preference information: %s (" SLAPI_COMPONENT_NAME_NSPR " error %d - %s)", 
+            val, errorCode, slapd_pr_strerror(errorCode));
+        slapi_ch_free((void **) &val);
+    }
+    freeConfigEntry(&e);
+
     /* Import pr fd into SSL */
     pr_sock = SSL_ImportFD( NULL, sock );
     if( pr_sock == (PRFileDesc *)NULL ) {
@@ -1555,8 +1668,6 @@ slapd_ssl_init2(PRFileDesc **fd, int startTLS)
         slapd_pk11_setSlotPWValues(slot, 0, 0);
     }
 
-
-
     /*
      * Now, get the complete list of cipher families. Each family
      * has a token name and personality name which we'll use to find
@@ -1739,9 +1850,8 @@ slapd_ssl_init2(PRFileDesc **fd, int startTLS)
             "out of disk space! Make more room in /tmp "
             "and try again. (" SLAPI_COMPONENT_NAME_NSPR " error %d - %s)",
             errorCode, slapd_pr_strerror(errorCode));
-      }
-      else {
-    slapd_SSL_error("Config of server nonce cache failed (error %d - %s)",
+      } else {
+        slapd_SSL_error("Config of server nonce cache failed (error %d - %s)",
             errorCode, slapd_pr_strerror(errorCode));
       }
       return rv;
@@ -1839,6 +1949,8 @@ slapd_ssl_init2(PRFileDesc **fd, int startTLS)
             } else {
                 enableTLS1 = slapi_entry_attr_get_bool( e, "nsTLS1" );
             }
+        } else if (enabledNSSVersions.max >= CURRENT_DEFAULT_SSL_VERSION) {
+            enableTLS1 = PR_TRUE; /* If available, enable TLS1 */
         }
         slapi_ch_free_string( &val );
 #if !defined(NSS_TLS10) /* NSS_TLS11 or newer */
@@ -1853,14 +1965,13 @@ slapd_ssl_init2(PRFileDesc **fd, int startTLS)
         }
         slapi_ch_free_string( &val );
         if (NSSVersionMin > NSSVersionMax) {
-            slapd_SSL_warn("Security Initialization: The min value of NSS version range "
-                        "\"%s\" is greater than the max value \"%s\"; "
-                           "the default range \"%s\" - \"%s\" is used.\n",
-                           getNSSVersion_str(NSSVersionMin), 
-                           getNSSVersion_str(NSSVersionMax),
-                           getNSSVersion_str(enabledNSSVersions.min),
-                           getNSSVersion_str(enabledNSSVersions.max));
-            NSSVersionMin = enabledNSSVersions.min;
+            (void) slapi_getSSLVersion_str(NSSVersionMin, mymin, sizeof(mymin));
+            (void) slapi_getSSLVersion_str(NSSVersionMax, mymax, sizeof(mymax));
+            slapd_SSL_warn("The min value of NSS version range \"%s\" is greater than the max value \"%s\".",
+                           mymin, mymax);
+            (void) slapi_getSSLVersion_str(enabledNSSVersions.max, newmax, sizeof(newmax));
+            slapd_SSL_warn("Reset the max \"%s\" to supported max \"%s\".",
+                           mymax, newmax);
             NSSVersionMax = enabledNSSVersions.max;
         }
 #endif
@@ -1868,18 +1979,21 @@ slapd_ssl_init2(PRFileDesc **fd, int startTLS)
 #if !defined(NSS_TLS10) /* NSS_TLS11 or newer */
     if (NSSVersionMin > 0) {
         /* Use new NSS API SSL_VersionRangeSet (NSS3.14 or newer) */
-        SSLVersionRange myNSSVersions;
-        myNSSVersions.min = NSSVersionMin;
-        myNSSVersions.max = NSSVersionMax;
-        restrict_SSLVersionRange(&myNSSVersions, enableSSL3, enableTLS1);
-        sslStatus = SSL_VersionRangeSet(pr_sock, &myNSSVersions);
+        slapdNSSVersions.min = NSSVersionMin;
+        slapdNSSVersions.max = NSSVersionMax;
+        restrict_SSLVersionRange();
+        (void) slapi_getSSLVersion_str(slapdNSSVersions.min, mymin, sizeof(mymin));
+        (void) slapi_getSSLVersion_str(slapdNSSVersions.max, mymax, sizeof(mymax));
+        slapi_log_error(SLAPI_LOG_FATAL, "SSL Initialization",
+                        "Configured SSL version range: min: %s, max: %s\n",
+                        mymin, mymax);
+        sslStatus = SSL_VersionRangeSet(pr_sock, &slapdNSSVersions);
         if (sslStatus == SECSuccess) {
             /* Set the restricted value to the cn=encryption entry */
         } else {
             slapd_SSL_error("SSL Initialization 2: "
                             "Failed to set SSL range: min: %s, max: %s\n",
-                            getNSSVersion_str(myNSSVersions.min),
-                            getNSSVersion_str(myNSSVersions.max));
+                            mymin, mymax);
         }
     } else {
 #endif
@@ -1948,17 +2062,17 @@ slapd_ssl_init2(PRFileDesc **fd, int startTLS)
 /* richm 20020227
    To do LDAP client SSL init, we need to do
 
-	static void
-	ldapssl_basic_init( void )
-	{
-    	PR_Init(PR_USER_THREAD, PR_PRIORITY_NORMAL, 0);
+    static void
+    ldapssl_basic_init( void )
+    {
+        PR_Init(PR_USER_THREAD, PR_PRIORITY_NORMAL, 0);
 
-    	PR_SetConcurrency( 4 );
-	}
+        PR_SetConcurrency( 4 );
+    }
     NSS_Init(certdbpath);
     SSL_OptionSetDefault(SSL_ENABLE_SSL2, PR_FALSE);
-	SSL_OptionSetDefault(SSL_ENABLE_SSL3, PR_TRUE);
-	s = NSS_SetDomesticPolicy(); 
+    SSL_OptionSetDefault(SSL_ENABLE_SSL3, PR_TRUE);
+    s = NSS_SetDomesticPolicy(); 
 We already do pr_init, we don't need pr_setconcurrency, we already do nss_init and the rest
 
 */   
@@ -1984,7 +2098,7 @@ slapd_SSL_client_auth (LDAP* ld)
         char **family;
         char *personality = NULL;
         char *activation = NULL;
-		char *cipher = NULL;
+        char *cipher = NULL;
 
         for (family = family_list; *family; family++) {
             getConfigEntry( *family, &entry );
