@@ -82,7 +82,7 @@ do_unbind( Slapi_PBlock *pb )
 	if ( ber_get_null( ber ) == LBER_ERROR ) {
 		slapi_log_access( LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d UNBIND,"
 				" decoding error: UnBindRequest not null\n",
-				(long long unsigned int)pb->pb_conn->c_connid, operation->o_opid );
+				pb->pb_conn->c_connid, operation->o_opid );
 		/* LDAPv3 does not allow a response to an unbind... so just return. */
 		goto free_and_return;
 	}
@@ -97,7 +97,7 @@ do_unbind( Slapi_PBlock *pb )
 	if ( (err = get_ldapmessage_controls_ext( pb, ber, NULL, ignore_criticality )) != 0 ) {
 		slapi_log_access( LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d UNBIND,"
 				" error processing controls - error %d (%s)\n",
-				(long long unsigned int)pb->pb_conn->c_connid, operation->o_opid,
+				pb->pb_conn->c_connid, operation->o_opid,
 				err, ldap_err2string( err ));
 		/* LDAPv3 does not allow a response to an unbind... so just return. */
 		goto free_and_return;
@@ -111,7 +111,7 @@ do_unbind( Slapi_PBlock *pb )
 	/* ONREPL - plugins should be called and passed bind dn and, possibly, other data */
 
 	slapi_log_access( LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d UNBIND\n",
-			(long long unsigned int)pb->pb_conn->c_connid, operation->o_opid );
+			pb->pb_conn->c_connid, operation->o_opid );
 
 	/* pass the unbind to all backends */
 	be_unbindall( pb->pb_conn, operation );
